/**
 * \file gre/labelgenerator.h
 * \brief GRE::LABELGENERATOR definitions
 *
 * \if NODOC
 * $Id: labelgenerator.h_v 1.2 2012/05/29 13:48:23 mju Exp $
 *
 * $Log: labelgenerator.h_v $
 * Revision 1.2  2012/05/29 13:48:23  mju
 * Use autolabelparms enums for places.
 *
 * Revision 1.1  2012/05/22 15:07:48  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LABELGENERATOR_H
#define	INC_GRE_LABELGENERATOR_H

#ifndef	INC_GRE_LABELPLACEMENT_H
#include <gre/labelplacement.h>
#endif

#ifndef	INC_GRE_AUTOLABELPARMS_H
#include <gre/autolabelparms.h>
#endif

#ifndef	INC_MI32_ELEMSTYL_H
#include <mi32/elemstyl.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class REGION2D;
class SUBREGION2D;
namespace MGD {
	class CONTEXT;
	}
#endif


namespace GRE {

//! Label generator.
//! Provides facility to generate labels for point, line and area (polygon) features.
class LABELGENERATOR {
	public:

		//! Default constructor.
		LABELGENERATOR ();

		//! Destructor.
		~LABELGENERATOR ();

		//! Generate label(s) for areas.
		//! Note that the returned label(s) will not have any specific text style or frame parameters set.
		ERRVALUE GenerateAreaLabels (
			const REGION2D& rgnArea,						//!< Region
			const MISTRING& Text,							//!< Label text string
			LABELPLACELIST& placelist						//!< Placement to add labels to
			) const;

		//! Generate label(s) for element described by region.
		//! Region must only contain one element type (point, line or subregion).
		//! Note that the returned label(s) will not have any specific text style or frame parameters set.
		ERRVALUE GenerateLabels (
			const REGION2D& rgnElement,					//!< Region describing element
			const MISTRING& Text,							//!< Label text string
			LABELPLACELIST& placelist						//!< Placement to add labels to
			) const;

		//! Generate label(s) for single line.
		//! Note that the returned label(s) will not have any specific text style or frame parameters set.
		ERRVALUE GenerateLineLabels (
			const POLYLINE& Line,							//!< Line
			const MISTRING& Text,							//!< Label text string
			LABELPLACELIST& placelist						//!< Placement to add labels to
			) const;

		//! Generate label(s) for one or more lines.
		//! Note that the returned label(s) will not have any specific text style or frame parameters set.
		ERRVALUE GenerateLineLabels (
			const POLYLINELIST& LineList,					//!< Lines
			const MISTRING& Text,							//!< Label text string
			LABELPLACELIST& placelist						//!< Placement to add labels to
			) const;

		//! Generate label for single point.
		//! Note that the returned label(s) will not have any specific text style or frame parameters set.
		ERRVALUE GeneratePointLabel (
			const DPOINT2D& point,							//!< Point location
			const MISTRING& Text,							//!< Label text string
			LABELPLACELIST& placelist						//!< Placement to add labels to
			) const;

		//! Generate label(s) for one or more points.
		ERRVALUE GeneratePointLabels (
			const POLYLINE& Points,							//!< Points
			const MISTRING& Text,							//!< Label text string
			LABELPLACELIST& placelist						//!< Placement to add labels to
			) const;

		//! Get current area label placelist setting.
		//! @return The current AUTOLABELPARMS::PLACE_AREA
		AUTOLABELPARMS::PLACE_AREA GetAreaPlace () const
			{ return (m_AreaPlace); }

		//! Get label baseline curvature setting for line label generation.
		//! @return Current line label baseline curvature setting.
		MGD::BASELINE GetLineBaseFit () const
			{ return (m_LineBaseFit); }

		//! Get minimum line length for generating label.
		double GetLineMinLength () const
			{ return (m_LineMinLength); }
			
		//! Get current line label placelist setting.
		//! @return The current AUTOLABELPARMS::PLACE_LINE
		AUTOLABELPARMS::PLACE_LINE GetLinePlace () const
			{ return (m_LinePlace); }

		//! Get spacing between labels for large polygons and long lines.
		//! @return Label spacing in object units.
		double GetMultiLabelSpacing () const
			{ return (m_MultiLabelSpacing); }

		//! Set current polygon label positioning setting.
		void SetAreaPlace (
			AUTOLABELPARMS::PLACE_AREA AreaPlace
			) { m_AreaPlace = AreaPlace; }

		//! Set how line baseline fits element.
		void SetLineBaseFit (
			MGD::BASELINE LineBaseFit
			) { m_LineBaseFit = LineBaseFit; }
			
		//! Set minimum line length necessary to generate a label.
		void SetLineMinLength (
			double LineMinLength						//!< Length in pixels
			) { m_LineMinLength = LineMinLength; }

		//! Set offset between text and line element.
		//! If not set (or reset to 0), offset will be 10% of text height.
		void SetLineOffset (
			double LineOffset							//!< Offset in pixels
			) { m_LineOffset = LineOffset; }
			
		//! Set line label place.
		void SetLinePlace (
			AUTOLABELPARMS::PLACE_LINE LinePlace
			) { m_LinePlace = LinePlace; }

		//! Set spacing between labels for large polygons and long lines
		//! Set to '0.0' to disable multilabel generation (default).
		void SetMultiLabelSpacing (
			double MultiLabelSpacing
			) { m_MultiLabelSpacing = MultiLabelSpacing; }

		//! Set offset between text and point element.
		//! If not set (or reset to 0), offset will be 10% of text height.
		void SetPointOffset (
			double PointOffset							//!< Offset in pixels
			) { m_PointOffset = PointOffset; }
			
		//! Set point label place.
		void SetPointPlace (
			AUTOLABELPARMS::PLACE_POINT PointPlace
			) { m_PointPlace = PointPlace; }

		//! Set text height used to determine extent.
		void SetTextHeight (
			double TextHeight						//!< Text height in pixels
			) { m_TextHeight = TextHeight; }

		//! Set text style used to determine extent.
		//! Note that the specified TextStyle is only used to determine the label extent.
		void SetTextStyle (
			const TEXTSTYLE& TextStyle
			) { m_TextStyle = TextStyle; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		mutable POLYLINE m_TempLine;
		MGD::CONTEXT *m_WorkGC;
		double m_MultiLabelSpacing;
		double m_LineMinLength;
		double m_LineOffset;
		double m_PointOffset;
		double m_TextHeight;
		TEXTSTYLE m_TextStyle;
		MGD::BASELINE m_LineBaseFit;
		GRE::AUTOLABELPARMS::PLACE_AREA m_AreaPlace;
		GRE::AUTOLABELPARMS::PLACE_LINE m_LinePlace;
		GRE::AUTOLABELPARMS::PLACE_POINT m_PointPlace;
		
		void AddLineLabel (const POLYLINE& line, const DPOINT3D& BasePoint, const MISTRING& Text, const DPOINT2D& TextSize, LABELPLACELIST& placelist) const;
		void AddLineLabels (const POLYLINE& line, const MISTRING& Text, const DPOINT2D& TextSize, LABELPLACELIST& placelist) const;
		void AddPointLabel (const DPOINT2D& point, const MISTRING& Text, LABELPLACELIST& placelist) const;
		ERRVALUE AddSubRegionLabels (const SUBREGION2D& subregion, const MISTRING& Text, const DPOINT2D& TextSize, LABELPLACELIST& placelist) const;
		ERRVALUE ComputeTextSize (const MISTRING& Text, DPOINT2D& TextSize) const;
			
		LABELGENERATOR (const LABELGENERATOR&);
		LABELGENERATOR& operator=(const LABELGENERATOR&);
		#endif
	};	// End LABELGENERATOR

}	// End namespace GRE

#endif	// INC_GRE_LABELGENERATOR_H
