/**
 * \file gre/lrsurface.h
 * \brief GRE LAYER_SURFACE definitions.
 *
 * \if NODOC
 * $Id: lrsurface.h_v 1.32 2013/01/22 18:38:04 mju Exp $
 *
 * $Log: lrsurface.h_v $
 * Revision 1.32  2013/01/22 18:38:04  mju
 * Add insertFilterValueScale.
 *
 * Revision 1.31  2012/08/16 16:27:44  mju
 * Add valueScale/Offset members to keep track of stored scaling.
 *
 * Revision 1.30  2012/07/23 22:45:23  bcolombe
 * Add unitshown to SetDatatip
 *
 * Revision 1.29  2012/06/12 21:44:25  mju
 * Add datatip methods.
 *
 * Revision 1.28  2012/05/03 21:44:12  mju
 * Add dispparm members for rendering.
 *
 * Revision 1.27  2012/04/30 20:09:45  mju
 * Add dispparm get/setColorRange... methods.
 *
 * Revision 1.26  2012/04/30 17:33:48  mju
 * Add dispparm members for color and mode enum.
 *
 * Revision 1.25  2012/04/30 15:08:08  mju
 * Add dispparm::setGeorefName.
 *
 * Revision 1.24  2012/04/27 21:13:56  mju
 * Make dispparm members private and add some needed for rendering.
 *
 * Revision 1.23  2012/03/09 16:33:09  mju
 * Add members for keeping section buffer for reading z values.
 *
 * Revision 1.22  2012/02/03 21:57:19  mju
 * Add createDefault/FromOnline and dlgGetObject static methods.
 *
 * Revision 1.21  2012/01/26 21:02:31  mju
 * Add getImageStageTerrain.
 *
 * Revision 1.20  2012/01/26 18:48:44  mju
 * Add canReadValues.
 *
 * Revision 1.19  2012/01/23 21:46:56  mju
 * Add stageTerrain member.
 *
 * Revision 1.18  2011/12/01 22:25:51  mju
 * Add v_constructLayerSection.
 *
 * Revision 1.17  2011/11/21 16:42:59  mju
 * Add v_estimateRangeZ.
 *
 * Revision 1.15  2010/10/07 16:37:19  mju
 * Add getPointZInterpolated and readCellZ.
 *
 * Revision 1.9  2006/09/08 20:25:27  mju
 * Add wireframe member.
 *
 * Revision 1.5  2005/06/17 17:06:29  vdronov
 * added saving of optimization structure
 *
 * Revision 1.1  2005/01/20 16:57:42  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRSURFACE_H
#define	INC_GRE_LRSURFACE_H

#ifndef	INC_GRE_LAYER_H
	#include <gre/layer.h>
#endif

#ifndef	INC_GRE_DATATIP_H
	#include <gre/datatip.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_COLORSPREAD_H
#include <mi32/colorspread.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class RASTER;
	class NULLMASK;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		class STAGE_AUTOPTR;
		class BUFFER;
		}
	}
#endif

namespace GRE {
//===================================================================================================================

//! Surface layer, used to drape other layers for 3D / stereo.
class LAYER_SURFACE : public LAYER {
	public:

		enum RENDERING_MODE {
			RENDERING_MODE_None =						  -1,
			RENDERING_MODE_VariableTriangulation =		0,
			RENDERING_MODE_PiecewiseTIN =					1,
			RENDERING_MODE_FlatTriangulation =			2
			};

		//! Surface layer display parameters.
		class DISPPARM : public LAYER::DISPPARM {
			public:

				class DLG;

				enum DISPLAYMODE {
					DISPLAYMODE_None =				0,		// Not a valid mode, set when loading so will hide terrain layers from old layouts
					DISPLAYMODE_ShadedRelief =		1,
					DISPLAYMODE_ColorElevation =	2,
					DISPLAYMODE_ColorShaded =		3,
					};

				enum COLORRANGEMODE {
					COLORRANGEMODE_Object =		0,
					COLORRANGEMODE_View = 		1,
					COLORRANGEMODE_Manual =		2
					};

				//! Default constructor.
				DISPPARM ();

				//! Copy constructor.
				DISPPARM (
					const DISPPARM& rhs
					): LAYER::DISPPARM(rhs) { Copy(rhs); }

				virtual ~DISPPARM (
					) { Free(); }

				//! Assignment.
				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

				double Get3dErrorTolerance () const
					{ return (m_3dErrorTolerance); }

				bool Get3dSaveTriangulation () const
					{ return (m_b3dSaveTriangulation); }

				INT32 Get3dWireframeSampling () const
					{ return (m_3dWireframeSampling); }

				double GetColorRangeMaximum () const
					{ return (m_ColorRangeMaximum); }

				double GetColorRangeMinimum () const
					{ return (m_ColorRangeMinimum); }

				COLORRANGEMODE GetColorRangeMode () const
					{ return (m_ColorRangeMode); }

				const COLORSPREAD& GetColorSpread () const
					{ return (m_ColorSpread); }

				const DATATIPPARMS& GetDataTip () const
					{ return (m_DataTip); }

				DISPLAYMODE GetDisplayMode () const
					{ return (m_DisplayMode); }

				const RVC::OBJECTNAME& GetGeorefName () const
					{ return (m_GeorefName); }

				const RVC::OBJITEM& GetObjItem () const
					{ return (m_ObjItem); }

				double GetShadingAzimuthAngle () const
					{ return (m_ShadingAzimuthAngle); }

				double GetShadingElevationAngle () const
					{ return (m_ShadingElevationAngle); }

				double GetShadingZScale () const
					{ return (m_ShadingZScale); }

				void Set3dErrorTolerance (
					double ErrorTolerance
					) { m_3dErrorTolerance = ErrorTolerance; }

				void Set3dSaveTriangulation (
					bool SaveTriangulation
					) { m_b3dSaveTriangulation = SaveTriangulation; }

				void Set3dWireframeSampling (
					INT32 WireframeSampling
					) { m_3dWireframeSampling = WireframeSampling; }

				void SetColorRange (double ColorRangeMinimum, double ColorRangeMaximum)
					{ m_ColorRangeMinimum = ColorRangeMinimum; m_ColorRangeMaximum = ColorRangeMaximum; }

				void SetColorRange (const DOUBLE_RANGE& ColorRange)
					{ SetColorRange(ColorRange.GetMinimum(),ColorRange.GetMaximum()); }

				void SetColorRangeMode (COLORRANGEMODE ColorRangeMode)
					{ m_ColorRangeMode = ColorRangeMode; }

				void SetColorSpread (const COLORSPREAD& ColorSpread)
					{ m_ColorSpread = ColorSpread; }

				void SetDataTip (
					bool visible,
					const MISTRING& prefix,
					const MISTRING& suffix,
					UINT8 decplaces,
					UINT32 unitshown
					);

				void SetDataTip (
					const DATATIPPARMS& datatip
					);

				void SetDataTipDefault ();

				void SetDisplayMode (DISPLAYMODE DisplayMode)
					{ m_DisplayMode = DisplayMode; }

				//! Set georeference object name.
				void SetGeorefName (
					const RVC::OBJECTNAME& GeorefName
					) { m_GeorefName = GeorefName; }

				ERRVALUE SetObjItem (
					const RVC::OBJITEM& ObjItem
					);

				//! Set shading azimuth angle.
				void SetShadingAzimuthAngle (
					double Azimuth					//!< Azimuth in radians
					) { m_ShadingAzimuthAngle = Azimuth; }

				//! Set shading elevation angles.
				void SetShadingElevationAngle (
					double Elevation				//!< Elevation in radians (0 - pi/2)
					) { m_ShadingElevationAngle = Elevation; }

				void SetShadingZScale (
					double ZScale
					) { if (ZScale > 0) m_ShadingZScale = ZScale; }

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				typedef LAYER_SURFACE::DISPPARM THISCLASS;

				RVC::OBJITEM m_ObjItem;
				RVC::OBJECTNAME m_GeorefName;				// Name of georeference object
				DATATIPPARMS m_DataTip;						// DataTip parameters
				COLORSPREAD m_ColorSpread;
				double m_ShadingAzimuthAngle;
				double m_ShadingElevationAngle;
				double m_ShadingZScale;
				double m_3dErrorTolerance;
				double m_ColorRangeMinimum;
				double m_ColorRangeMaximum;
				INT32 m_3dWireframeSampling;				// 3D Wireframe sampling rate
				DISPLAYMODE m_DisplayMode;
				COLORRANGEMODE m_ColorRangeMode;
				bool m_b3dSaveTriangulation;

				void Copy (const DISPPARM&);
				void Free ();
				void Init ();

				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_SURFACE::DISPPARM

		//! Create default surface layer, prompting user or using from online terrain.
		static ERRVALUE CreateDefault (
			MDLGPARENT dlgparent,								//!< Parent for dialog when prompt user
			GROUP *group,											//!< Group
			LAYER_SURFACE **pLayer = 0							//!< Returned pointer to layer created if needed
			);

		//! Create default surface layer from online terrain.
		static ERRVALUE CreateDefaultFromOnline (
			GROUP *group,											//!< Group
			LAYER_SURFACE **pLayer = 0							//!< Returned pointer to layer created if needed
			);

		//! Prompt user to select object for terrain layer.
		static ERRVALUE DlgGetObject (
			MDLGPARENT dlgparent,								//!< Parent for dialog when prompt user
			RVC::OBJITEM& ObjItem								//!< Object location returned
			);

		//! Register this layer type
		static void RegisterType ();

		//! Construct layer and insert into list
		explicit LAYER_SURFACE (
			GRE::GROUP *group,									//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0									//!< Reference layer for insertion
			);

		virtual ~LAYER_SURFACE ();

		//! Determine if can read surface values directly.
		//! Use to determine if can call any of the GetPoint/sZ or Read methods.
		bool CanReadValues () const
			{ return (m_dispparm.GetObjItem().GetObjectType() == RVC::OBJTYPE_Raster); }

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm () const
			{ return (m_dispparm); }

		//! Get Z value for single point.
		//! @return TRUE if valid, FALSE if not, < 0 if error
		int GetPointZ (
			const DPOINT2D& spoint,						//!< Point in surface object coordinates
			double& z,										//!< Z value returned if return value indicates valid, unchanged if not
			bool DoSearch									//!< Do hueristic neighborhood search if point just off edge or on null cell
			);

		//! Get Z value for single point.
		//! @return TRUE if valid, FALSE if not, < 0 if error
		int GetPointZInterpolated (
			const DPOINT2D& spoint,						//!< Point in surface object coordinates
			double& z										//!< Z value returned if return value indicates valid, unchanged if not
			);

		//! Get Z coordinates for set of points.
		//! This method is used mainly in 3D rendering and measurement to obtain the Z coordinates
		//! of one or more points in some other map projection or layer coordinate system.
		int GetPointsZ (
			const TRANS2D_MAPGEN& PointToSurface,	//!< Transformation from point to surface object coordinates
			DPOINT3D *points,								//!< Points to obtain Z values for
			INT32 NumPoints,								//!< Number of points
			double dftz										//!< Default Z value for off-surface points
			);

		//! Get Z coordinates for set of points.
		//! This method is used mainly in 3D rendering and measurement to obtain the Z coordinates
		//! of one or more points in some other map projection or layer coordinate system.
		ERRVALUE GetPointsZ (
			const TRANS2D_MAPGEN& PointToSurface,	//!< Transformation from point to surface object coordinates
			POLYLINE& PolyLine,							//!< Points to obtain Z values for
			double dftz										//!< Default Z value for off-surface points
			);

		//! Get Z coordinates for set of points with Bilinear interpolation.
		//! This method is used mainly in 3D editing to obtain the Z coordinates
		//! of one or more points in some other layer coordinate reference system.
		ERRVALUE GetPointsZBilinear (
			const TRANS2D_MAPGEN& PointToSurface,	//!< Transformation from point to surface object coordinates
			POLYLINE& PolyLine,							//!< Points to obtain Z values for
			double dftz										//!< Default Z value for off-surface points
			);

		//! Return pointer to image stage for terrain.
		//! Do not delete, or assign ownership for, the returned pointer.
		SPATMOD::IMAGE::STAGE* GetImageStageTerrain () const
			{ return (m_pStageTerrain); }

		GRE::TERRAIN* GetTerrain (
			) { return (m_pTerrain); };

		//! Read from surface raster.
		ERRVALUE Read (
			INT32 row,										//!< Initial row
			INT32 col,										//!< Initial column
			INT32 ncols,									//!< Number of columns
			double *buffer,								//!< Buffer for result
			UINT8 *mask										//!< Buffer for mask, 0 if don't need
			);

		//! Set layer-specific display parameters.
		//! Validation will be performed on the specified parameters and adjustments
		//! made to the layer's copy if necessary.  Use of this method usually results
		//! in a ChangeBegin / ChangeEnd message pair, and will also generate a
		//! ChangeObject message if the spatial object used in the layer is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::RASTER *m_pObjRaster;
		RVC::NULLMASK *m_pObjMask;
		GRE::TERRAIN *m_pTerrain;
		GRE::WIREFRAME *m_pWireframe;
		SPATMOD::IMAGE::STAGE *m_pStageTerrain;
	#ifdef USE_MGUI
		DISPPARM::DLG *m_pDlgControlPanel;
	#endif
		mutable SPATMOD::IMAGE::BUFFER *m_pBuffer;
		mutable LRECT2D m_RectBuffer;
		double m_ValueScale;
		double m_ValueOffset;

		void CalcWireSampleRange (INT32& WireSampleMin, INT32& WireSampleMax) const;
		void CheckTerrain ();
		ERRVALUE Draw2D (LAYERDC& layerdc);
		ERRVALUE Draw3D (LAYERDC& layerdc);
		void FreeStageTerrain ();
		ERRVALUE InsertFilterValueScale (SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStage) const;
		int ReadCellZ (INT32 col, INT32 row, double& z) const;
		ERRVALUE SetupNewObject (const RVC::OBJITEM& ObjItem);
		ERRVALUE SetupStageSource (SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStage) const;
		int WireReadRaster ();

		// Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual void v_CloseObject ();
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW *view);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual void v_DataTipGetDefaultPrefix (int component, DATATIPPARMS& DataTipParms) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual int v_DataTipGetString (const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images, GRE::VIEW* view);
		virtual void v_DataTipSetDefault (int component, bool force);
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_EstimateRangeZ (const LAYERVIEW *pLayerView, DOUBLE_RANGE& range) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual double v_GetScaleMaxDetail (const DPOINT2D& ptTgt, const SPATREF::COORDREFSYS& crsTgt, double DisplayPixelSizeMM) const;
		virtual int v_OpenObject (MDLGPARENT dlgparent);
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_SURFACE (const LAYER_SURFACE&);
		LAYER_SURFACE& operator= (const LAYER_SURFACE&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRSURFACE_H
