/**
 * \file mgui/base.h 
 * \brief MGUI base header
 *
 * \if NODOC
 * $Id: base.h_v 1.18 2012/01/27 18:33:31 mju Exp $
 *
 * $Log: base.h_v $
 * Revision 1.18  2012/01/27 18:33:31  mju
 * Remove unused fwd decl.
 *
 * Revision 1.17  2012/01/27 16:20:18  mju
 * Add delegate_ontooltiprequest.
 *
 * Revision 1.14  2011/08/19 14:24:53  mju
 * Add point::setX/Y.
 *
 * Revision 1.8  2009/12/10 22:08:28  mju
 * Add point in/equality operators.
 *
 * Revision 1.1  2005/06/22 15:31:05  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_BASE_H
#define	INC_MGUI_BASE_H

#ifndef INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

namespace MGUI {
//===================================================================================================================

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class FORM;
class CTRL;
class CTRL_TOOLTIP;
class MENU;
class LAYOUT_PANE_BASE;
#endif

//===================================================================================================================
//! Class to contain X/Y coordinates of a point in a window, on the screen, etc.
//! Often instances of this class are passed by value since the total size is 4 bytes
//! which actually results in smaller and more efficient code in the called function
//! due to direct access on stack rather than through pointer/reference indirection.
class POINT {
	public:

		INT16 m_x;								//!< Point X coordinate
		INT16 m_y;								//!< Point Y coordinate

		//! Default construction, initializes to arbitrary point.
		POINT (
			): m_x(-32768), m_y(-32768) { }

		POINT (
			const POINT& rhs
			): m_x(rhs.m_x), m_y(rhs.m_y) { }

		//! Construct from X, Y values.
		POINT (
			INT16 x,
			INT16 y
			): m_x(x), m_y(y) { }

		POINT& operator= (
			const POINT& rhs
			) { if (this != &rhs) { m_x = rhs.m_x; m_y = rhs.m_y; } return (*this); }

		//! Get X value.
		INT16 GetX () const
			{ return (m_x); }

		//! Get Y value.
		INT16 GetY () const
			{ return (m_y); }

		//! Set X, Y values.
		void Set (
			INT16 x,
			INT16 y
			) { m_x = x; m_y = y; }

		//! Set X value.
		void SetX (
			INT16 x
			) { m_x = x; }

		//! Set Y value.
		void SetY (
			INT16 y
			) { m_y = y; }
	};

inline bool operator== (
	const MGUI::POINT& lhs,
	const MGUI::POINT& rhs
	) { return (lhs.m_x == rhs.m_x && lhs.m_y == rhs.m_y); }

inline bool operator!= (
	const MGUI::POINT& lhs,
	const MGUI::POINT& rhs
	) { return (lhs.m_x != rhs.m_x || lhs.m_y != rhs.m_y); }

//===================================================================================================================


enum LAYOUT_ORIENTATION {
	LAYOUT_ORIENTATION_Horizontal =		0,		//!< Horizontal layout, automatic LtoR/RtoL based on locale
	LAYOUT_ORIENTATION_Vertical =			1,		//!< Vertical top-to-bottom layout
	LAYOUT_ORIENTATION_HorizontalLtoR =	2,		//!< Horizontal left-to-right layout
	LAYOUT_ORIENTATION_HorizontalRtoL =	3,		//!< Horizontal right-to-left layout
	};

enum LAYOUT_SIZEALIGN {
	LAYOUT_SIZEALIGN_Expand =					0x0000,	//!< Expand both horizontal and vertical to fill space
	LAYOUT_SIZEALIGN_FixedWidth =				0x0001,	//!< Fixed width, height can expand
	LAYOUT_SIZEALIGN_RelativeWidth =			0x0002,	//!< Relative width as percentage of available space
	LAYOUT_SIZEALIGN_FixedHeight =			0x0004,	//!< Fixed height, width can expand
	LAYOUT_SIZEALIGN_RelativeHeight =		0x0008,	//!< Relative height as percentage of available space
	LAYOUT_SIZEALIGN_FixedSize =				0x0005,	//!< Both width and height fixed
	LAYOUT_SIZEALIGN_AlignLeft =				0x0010,	//!< Align with left edge of available space
	LAYOUT_SIZEALIGN_AlignRight =				0x0020,	//!< Align with right edge of available space
	LAYOUT_SIZEALIGN_AlignTop =				0x0040,	//!< Align with top edge of available space
	LAYOUT_SIZEALIGN_AlignBottom =			0x0080,	//!< Align with bottom edge of available space
	LAYOUT_SIZEALIGN_CenterHorz =				0x0030,	//!< Center horizontally in available space
	LAYOUT_SIZEALIGN_CenterVert =				0x00C0,	//!< Center vertically in available space
	LAYOUT_SIZEALIGN_Center =					0x00F0,	//!< Center both horizontal and vertically
	LAYOUT_SIZEALIGN_ExcludeFromLayout =	0x0100,	//!< Exclude from layout, will be placed at top/left unless other Align/Center values are set
	LAYOUT_SIZEALIGN_NoResizeAfterCreate =	0x2000,	//!< Don't resize pane after create (ie, when change managed children)
	LAYOUT_SIZEALIGN_IgnoreNonVisible =		0x4000,	//!< Ignore nonvisible items in layout
	LAYOUT_SIZEALIGN_InitiallyHidden =		0x8000,	//!< Create initially hidden, if supported
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(LAYOUT_SIZEALIGN);
#endif

//===================================================================================================================

//! Make a beep/bell sound.
void Beep ();

//===================================================================================================================

//! Delegate for function returning void with MGUI::ID parameter.
typedef fastdelegate::FastDelegate<void(MGUI::ID)> DELEGATE_VOID_ID;

//! Delegate to create custom controls in standard dialog.
typedef fastdelegate::FastDelegate<void(MGUI::LAYOUT_PANE_BASE&)> DELEGATE_CREATECUSTOMCONTROLS;

//! Delegate to call when requesting to show ToolTip.
//! Return true to show, false to not.
//! Parameters are: reference to tooltip instance and pointer x, y.
typedef fastdelegate::FastDelegate<bool(CTRL_TOOLTIP&, int, int)> DELEGATE_ONTOOLTIPREQUEST;

//===================================================================================================================
}	// End namespace MGUI

#endif	 // INC_MGUI_BASE_H
