/** 
 * \file mgui/formextents.h 
 * \brief Definitions for MGUI::FORM_EDIT_EXTENTS class.
 *
 * \if NODOC
 * $Id: formextents.h_v 1.12 2011/03/25 18:43:50 mju Exp $
 *
 * $Log: formextents.h_v $
 * Revision 1.12  2011/03/25 18:43:50  mju
 * Add flag_Precision0/minimalWidth.
 *
 * Revision 1.11  2007/07/09 19:20:29  mju
 * Add flag_noShowCRS.
 *
 * Revision 1.10  2007/03/28 19:49:03  drb
 * Added/fixed some editable/enabled methods
 *
 * Revision 1.8  2005/06/24 17:49:40  mju
 * Support delegates, don't use mgui templates.
 *
 * Revision 1.1  2004/06/10 21:28:09  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMEXTENTS_H
#define	INC_MGUI_FORMEXTENTS_H

#ifndef	INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef	INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

namespace MGUI {

//===================================================================================================================
//! Form consisting of label and string edit field.
class FORM_EDIT_EXTENTS : public MGUI::FORM_COMPOSITE {
	public:

		enum FLAGS {
			FLAG_None =								0x00,
			FLAG_AllowAllGeorefDatumTypes =	0x01,	//!< Allow all georeference-supported datum types
			FLAG_NoChangeCoordRefSys =			0x02,	//!< Disallow changing of coordinate reference system
			FLAG_NoShowCRS =						0x04,	//!< Don't show CRS selection controls
			FLAG_MinimalWidth =					0x08,	//!< Use minimal width for form
			FLAG_Precision0 =						0x10,	//!< Use 0 decimal places of precision
			};
	
		//! Constructor.
		FORM_EDIT_EXTENTS (
			);

		//! Destructor.
		virtual ~FORM_EDIT_EXTENTS (
			);

		//! Clear range edit controls.
		void ClearExtents (
			bool notify = true
			);

		//! Create form with label from resource lookup.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			bool IsRangeEditable = true,
			FLAGS flags = FLAG_None,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get the extents of the form
		const DRECT2D& GetExtents (
			) const { return (m_Extents); }
			
		//! Get the projection to use for extents selection.
		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const { return (m_CoordRefSys); }

		//! Determine if user has entered values for all range fields.
		bool HasAllRangeValues (
			);

		//! Set delegate to call when coordinate reference system changed.
		void SetDelegateOnChangeCoordRefSys (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeCoordRefSys = delegate; }

		//! Set delegate to call when extents change.
		void SetDelegateOnChangeExtents (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeExtents = delegate; }

		//! Set the extents of the form
		//! Do this after the call to SetCoordRefSys() or the call to the method will alter the extents
		void SetExtents (
			const DRECT2D& Extents,
			bool notify = true
			);

		//! Set the projection to use for extents selection.
		//! If a projection and value coordinates exists, the coordinates will be converted to the new projection
		void SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys,
			bool notify = true
			);

		//! Set the coordinate reference system button and text as editable.
		//! If not editable, button is disabled but text is still visible
		void SetCoordRefSysEditable (
			bool IsCRSEditable
			);

		//! Set the extents section to be editable
		void SetRangeEditable (
			bool IsRangeEditable
			);

		//! Set the extents section to be enabled
		void SetRangeEnabled (
			bool IsRangeEnabled
			);


	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SPATREF::COORDREFSYS m_CoordRefSys;
		DRECT2D m_Extents;
		bool m_IsRangeEnabled;
		bool m_IsRangeEditable;
		FLAGS m_flags;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeCoordRefSys;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeExtents;
		
		FORM_BUTTONTEXT m_BtxCoordRefSys;

		LAYOUT_PANE m_YPane;
		CTRL_LABEL m_YLabel;
		CTRL_EDIT_NUMBER m_YInit;
		CTRL_LABEL m_YTo;
		CTRL_EDIT_NUMBER m_YLast;
		
		LAYOUT_PANE m_XPane;
		CTRL_LABEL m_XLabel;
		CTRL_EDIT_NUMBER m_XInit;
		CTRL_LABEL m_XTo;
		CTRL_EDIT_NUMBER m_XLast;
		
		void NotifyChangeExtents ();
		void OnChangeExtentsValue ();
		void OnBtnCoordRefSys ();
		
		void UpdateProjection ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_EDIT_EXTENTS::FLAGS)
#endif

//===================================================================================================================

}	// End of MGUI namespace

#endif		//	INC_MGUI_FORMEXTENTS_H
