/**
 * \file mgui/menu.h
 * \brief MGUI::MENU class definitions.
 *
 * \if NODOC
 * $Id: menu.h_v 1.30 2013/01/23 14:49:04 mju Exp $
 *
 * $Log: menu.h_v $
 * Revision 1.30  2013/01/23 14:49:04  mju
 * Add hasItemID.
 *
 * Revision 1.29  2012/01/27 18:47:26  mju
 * Remove mfc stuff.
 *
 * Revision 1.27  2010/04/07 17:44:51  mju
 * Add addDelegateOnCreateWidget.
 *
 * Revision 1.24  2009/09/14 20:31:08  mju
 * Add popup variant using mgui::point.
 *
 * Revision 1.23  2008/07/24 21:37:43  scowan
 * Added a on wheel delegate for popup menus.
 *
 * Revision 1.21  2008/02/04 21:19:09  mju
 * Add deleteAll.
 *
 * Revision 1.20  2008/01/31 22:18:24  scowan
 * Added methods to set the label of a menu item.
 *
 * Revision 1.18  2007/12/17 15:29:37  mju
 * Add createitem_hidden and setVisible/ID.
 *
 * Revision 1.17  2007/12/07 18:21:12  mju
 * Add createitem_Check/RadioOn values and creatitem_Check/ToggleSet methods.
 *
 * Revision 1.14  2007/08/30 13:12:28  mju
 * Add getWidgetFromID.
 *
 * Revision 1.12  2007/03/07 22:15:43  mju
 * Add get/setToggleState.
 *
 * Revision 1.8  2005/10/28 20:40:29  mju
 * Support icons and toolbar menu type.
 *
 * Revision 1.7  2005/10/24 20:40:43  mju
 * Add setEnabledAll.
 *
 * Revision 1.5  2005/09/22 15:46:55  mju
 * Combine append and insert methods into add methods with default parm.
 * Add isEmpty.
 *
 * Revision 1.1  2005/09/09 16:58:24  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_MENU_H
#define	INC_MGUI_MENU_H

#ifndef	INC_MGUI_BASE_H
#include <mgui/base.h>
#endif

#ifndef	INC_MI32_ICONID_H
#include <mi32/iconid.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
#endif

namespace MGUI {

//=====================================================================================================================

//! Definition and manager for menu.
//! 
//!	To create a menubar and attach to a shell:
//!
//!	\code
//!	int MYSHELL::v_CreateContent (
//!		) {	
//!		// Create File menu (all choices call same delegate)
//!		MGUI::MENU MenuFile;
//!		MenuFile.Create(MGUI::MENU::TYPE_SubMenu,MGUI::MENU::DELEGATE(this,&MYSHELL::OnMenuFile));
//!		MenuFile.Add(TEXTID_Generic_New,ID_FileNew);
//!		MenuFile.Add(TEXTID_Generic_Open,ID_FileOpen);
//!		...
//!		MenuFile.Add(TEXTID_Generic_Exit,ID_FileExit);
//!		
//!		// Create View menu (each choice calls different delegate)
//!		MGUI::MENU MenuView;
//!		MenuView.Create(MGUI::MENU::TYPE_SubMenu);
//!		MenuView.Add(TEXTID_Generic_ZoomIn,MGUI::MENU::DELEGATE(this,&MYSHELL::OnMenuViewZoomIn));
//!		MenuView.Add(TEXTID_Generic_ZoomOut,MGUI::MENU::DELEGATE(this,&MYSHELL::OnMenuViewZoomOut));
//!		MenuView.Add(TEXTID_Generic_ZoomFull,MGUI::MENU::DELEGATE(this,&MYSHELL::OnMenuViewZoomFull));
//!		
//!		// Create the menubar
//!		MGUI::MENU MenuBar;
//!		MenuBar.Create(MGUI::MENU::TYPE_MenuBar);
//!		MenuBar.Add(TEXTID_Generic_File,MenuFile);
//!		MenuBar.Add(TEXTID_Generic_View,MenuView);
//!		
//!		// Attach menubar to shell
//!		AttachMenuBar(menubar);
//!		
//!		return (TRUE);
//!		}
//!	\endcode
//!
//! Note, MENU instances are reference-counted to allow enabling/disabling of menu items after
//! actual control creation.  For this reason, program must not attempt to use the same MENU instance
//! in more then one control or shell.
class MENU {
	public:

		//! Menu types.
		enum TYPE {
			TYPE_MenuBar,					//!< Menubar for use in SHELL
			TYPE_SubMenu,					//!< Submenu to add to menubar or another submenu
			TYPE_Popup,						//!< Pop-up menu
		#ifndef GENERATING_DOXYGEN_OUTPUT
			TYPE_ToolBar					//!< Used by CTRL_TOOLBAR, should not be used directly
		#endif
			};

		//! Flags for use when adding items to menu.
		enum CREATEITEM {
			CREATEITEM_Default =			0,
			CREATEITEM_Check =			0x0001,		//!< Create with check style, cannot be combined with CREATEITEM_Radio
			CREATEITEM_CheckOn =			0x0009,		//!< Create with check style, initially toggled on
			CREATEITEM_Radio =			0x0002,		//!< Create with radio style, cannot be combined with CREATEITEM_Check
			CREATEITEM_RadioOn =			0x000A,		//!< Create with radio style, initially toggled on
			CREATEITEM_RadioFirst =		0x0004,		//!< First item in new radio group, implies radio style, only need if multiple radio groups
			CREATEITEM_RadioFirstOn =	0x000C,		//!< First item in new radio group, initially toggled on
			CREATEITEM_ToggleOn =		0x0008,		//!< Create initially toggled on, requires CREATEITEM_Check or CREATEITEM_Radio
			CREATEITEM_Disabled =		0x0010,		//!< Create initially disabled
			CREATEITEM_NoDelegate =		0x0020,		//!< Create item with no delegate, use when menu has default delegate if want label
			CREATEITEM_MI_Internal =	0x0040,		//!< Only create if MicroImages internal license
			CREATEITEM_MI_Programmer =	0x0080,		//!< Only create if MicroImages programmer license
			CREATEITEM_Hidden =			0x0100		//!< Create item initially hidden
			};

		//! Flags for Popup().
		enum POPUP {
			POPUP_AlignLeft =				0x0001,		//!< Align left side of menu with specified X coordinate
			POPUP_AlignCenter =			0x0002,		//!< Align center of menu horizontally with specified X coordinate
			POPUP_AlignRight =			0x0004,		//!< Align right side of menu with specified X coordinate
			POPUP_MouseLeft =				0x0010,		//!< Allow selection via left mouse button
			POPUP_MouseRight =			0x0020,		//!< Allow selection via right mouse button
			POPUP_MouseEither =			0x0030,		//!< Allow selection via either mouse button
			POPUP_Default = POPUP_AlignLeft | POPUP_MouseEither
			};

		//! Menu item handle.
		enum ITEMHND {
			ITEMHND_None = -1,
			ITEMHND_Last = -2
			};

		//! Delegate for selection from menu.
		//! Values passed to delegate are ID and toggle state.  If item is not a toggle the
		//! state will always be 'true'.
		typedef fastdelegate::FastDelegate<void(MGUI::ID,bool)> DELEGATE;
		//! Delegate for mouse wheel movement on the menu
		//! Can be used to change state of the menu (TYPE_Popup)
		typedef fastdelegate::FastDelegate<void(INT32)> DELEGATE_ONWHEEL;

		//! Convenience method to get CREATEITEM flag for check item with control of whether initially on.
		static CREATEITEM CREATEITEM_CheckSet (
			bool ToggleOn
			) { return (ToggleOn ? CREATEITEM_CheckOn : CREATEITEM_Check); }

		//! Convenience method to get CREATEITEM flag for radio item with control of whether initially on.
		static CREATEITEM CREATEITEM_RadioSet (
			bool ToggleOn,
			bool RadioFirst = false
			) { return (RadioFirst ? (ToggleOn ? CREATEITEM_RadioFirstOn : CREATEITEM_RadioFirst) : (ToggleOn ? CREATEITEM_RadioOn : CREATEITEM_Radio)); }

		//! Default constructor.
		MENU ();

		//! Copy constructor.
		MENU (const MENU& rhs);

		//! Destructor.
		~MENU ();

		//! Assignment.
		MENU& operator= (const MENU& rhs);

		//! Add item to the menu, with label and default delegate.
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			const MISTRING& label,				//!< Label for item
			ID id,									//!< Application-defined ID to be passed to default delegate
			CREATEITEM createflags = CREATEITEM_Default,
			ITEMHND hndInsertAfter = ITEMHND_Last,	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add item to the menu, with icon, label and default delegate.
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			ICONID iconid,							//!< Icon ID
			const MISTRING& label,				//!< Label for item
			ID id,									//!< Application-defined ID to be passed to default delegate
			CREATEITEM createflags = CREATEITEM_Default,
			ITEMHND hndInsertAfter = ITEMHND_Last,	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add item to the menu, with label and specified delegate.
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			const MISTRING& label,				//!< Label for item
			MGUI::MENU::DELEGATE delegate,	//!< Delegate to call
			ID id = 0,								//!< Optional application-defined ID to be passed to delegate
			CREATEITEM createflags = CREATEITEM_Default,
			ITEMHND hndInsertAfter = ITEMHND_Last,	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add item to the menu, with icon, label and specified delegate.
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			ICONID iconid,							//!< Icon ID
			const MISTRING& label,				//!< Label for item
			MGUI::MENU::DELEGATE delegate,	//!< Delegate to call
			ID id = 0,								//!< Optional application-defined ID to be passed to delegate
			CREATEITEM createflags = CREATEITEM_Default,
			ITEMHND hndInsertAfter = ITEMHND_Last,	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add submenu item, with label.
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			const MISTRING& label,				//!< Label
			MENU& submenu,							//!< Submenu to add
			CREATEITEM createflags = CREATEITEM_Default,
			ITEMHND hndInsertAfter = ITEMHND_Last,	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add submenu item, with icon and label.
		//! @return Handle to item for subsequent reference, will be < 0 if error
		ITEMHND Add (
			ICONID iconid,							//!< Icon ID
			const MISTRING& label,				//!< Label
			MENU& submenu,							//!< Submenu to add
			CREATEITEM createflags = CREATEITEM_Default,
			ITEMHND hndInsertAfter = ITEMHND_Last,	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			UINT32 icid = 0						//!< Interface customization ID
			);

		//! Add separator to menu.
		ITEMHND AddSeparator (
			ITEMHND hndInsertAfter = ITEMHND_Last	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			);

		//! Add fixed space to menu.
		//! \internal Should only be used on menubar and by toolbar implementation.
		ITEMHND AddSpaceFixed (
			int space,								//!< Space in pixels to add
			ITEMHND hndInsertAfter = ITEMHND_Last	//!< Handle for item to insert after, ITEMHND_Last to insert at end
			);

		//! Create menu definition.
		//! This does not actually create the interface controls, that is usually done using MGUI::SHELL or Popup().
		//! May be called on previously created menu to reuse class instance.
		void Create (
			TYPE type								//!< Type of menu to create
			);

		//! Create menu definition with specified default delegate.
		//! May be called on previously created menu to reuse class instance.
		void Create (
			TYPE type,								//!< Type of menu to create
			MGUI::MENU::DELEGATE delegate		//!< Default delegate to use for items when none specified
			);

		//! Delete item from menu.
		void Delete (
			ITEMHND hndItem						//!< Handle for item to delete
			);

		//! Delete all items from menu.
		void DeleteAll (
			);

		//! Determine if has been created.
		bool IsCreated (
			) const { return (m_pPriv != 0); }

		//! Determine if menu is empty.
		bool IsEmpty (
			) const;

		//! Get state of Check or Radio item.
		//! Will return false if not a toggle item.
		bool GetToggleState (
			ITEMHND hndItem						//!< Handle returned by Append()
			) const;

		//! Get state of Check or Radio item.
		//! Will return false if not a toggle item.
		bool GetToggleStateID (
			ID id										//!< ID of item
			) const;

		//! Determine if menu contains item with specified ID.
		bool HasItemID (
			ID id										//!< ID of item
			) const;

		//! Display a popup menu and wait for user to make selection.
		//! @return ID of selected item if no delegate was called, 0 if delegate called or no selection made.
		ID Popup (
			CTRL& ctrl,								//!< Control which invoked popup action
			const LPOINT2D& point,				//!< Point where menu is to appear, relative to specified control
			POPUP popupflags = POPUP_Default	//!< Flags for positioning and selection
			);

		//! Display a popup menu and wait for user to make selection.
		//! @return ID of selected item if no delegate was called, 0 if delegate called or no selection made.
		ID Popup (
			CTRL& ctrl,								//!< Control which invoked popup action
			const MGUI::POINT& point,			//!< Point where menu is to appear, relative to specified control
			POPUP popupflags = POPUP_Default	//!< Flags for positioning and selection
			);

		//! Set delegate for existing item ID.
		void SetDelegate (
			ID id,									//!< ID of item
			DELEGATE delegate						//!< Delegate to set
			);

		//! Set delegate for existing menu when the pointer's wheel moves
		void SetDelegateOnPointerWheel (
			DELEGATE_ONWHEEL delegate			//!< Delegate to set
			);

		//! Set whether item is enabled or not, using ITEMHND
		void SetEnabled (
			ITEMHND hndItem,						//!< Handle returned by Append()
			bool enabled							//!< true to enable, false to disable
			);

		//! Set whether all items enabled or not.
		void SetEnabledAll (
			bool enabled							//!< true to enable, false to disable
			);

		//! Set whether item is enabled or not, using ID.
		void SetEnabledID (
			ID id,									//!< ID of item(s) to enable
			bool enabled							//!< true to enable, false to disable
			);

		//! Set menu item label, referenced by ITEMHND
		void SetLabel (
			ITEMHND hndItem,						//!< Handle returned by Append()
			const MISTRING& label				//!< Label
			);

		//! Set menu item label, referenced by ID
		void SetLabelID (
			ID id,									//!< ID of item(s) to show
			const MISTRING& label				//!< Label
			);

		//! Set state of toggle item by ITEMHND.
		//! If item is a "radio" button, previously selected item will be toggled off.
		void SetToggleState (
			ITEMHND hndItem,						//!< Handle returned by Append()
			bool ToggleOn,							//!< True to toggle on, false to toggle off
			bool notify								//!< Invoke delegate if state changed
			);

		//! Set state of toggle item by ID.
		//! If item is a "radio" button, previously selected item will be toggled off.
		void SetToggleStateID (
			ID id,									//!< ID of item
			bool ToggleOn,							//!< True to toggle on, false to toggle off
			bool notify								//!< Invoke delegate if state changed
			);

		//! Set whether item is visible or not, using ITEMHND
		void SetVisible (
			ITEMHND hndItem,						//!< Handle returned by Append()
			bool visible							//!< true to show, false to hide
			);

		//! Set whether item is visible or not, using ID.
		void SetVisibleID (
			ID id,									//!< ID of item(s) to show
			bool visible							//!< true to show, false to hide
			);

	#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;

		typedef fastdelegate::FastDelegate<void(Widget)> DELEGATE_ONCREATEWIDGET;

		// Add delegate to call when menu widget is actually created.
		void AddDelegateOnCreateWidget (
			DELEGATE_ONCREATEWIDGET delegate
			);

		// Create menubar for X, called from MGUI::SHELL::AttachMenuBar().
		void CreateMenuBarX (
			LAYOUT_PANE_BASE& parentpane
			);

		// Create toolbar for X, called from MGUI::CTRL_TOOLBAR::Create()
		Widget CreateToolBarX (
			LAYOUT_PANE_BASE& parentpane,
			LAYOUT_ORIENTATION orientation,
			LAYOUT_SIZEALIGN sizealign
			);

		// Get internal private pointer, called by DLG_CUSTOMIZE.
		PRIV* GetPrivPtr (
			) const { return (m_pPriv); }

		// Get X Widget from item ID.
		// For internal MGUI use only.
		Widget GetWidgetFromID (
			ID id
			) const;

	private:
		PRIV *m_pPriv;
	#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MENU::CREATEITEM)
DEFINE_ENUM_OP_BITWISE(MENU::POPUP)
#endif

//=====================================================================================================================
}	//! End namespace MGUI

#endif	// INC_MGUI_MENU_H
