/**
 * \file mgui/toolpoint.h
 * \brief MGUI::TOOL_POINT definitions
 *
 * \if NODOC
 * $Id: toolpoint.h_v 1.15 2011/12/09 20:23:22 mju Exp $
 *
 * $Log: toolpoint.h_v $
 * Revision 1.15  2011/12/09 20:23:22  mju
 * No context in v_draw.
 *
 * Revision 1.14  2011/08/18 15:45:46  mju
 * Add support for drawing "symbol" based on style.
 *
 * Revision 1.13  2011/06/21 14:13:24  mju
 * *** empty log message ***
 *
 * Revision 1.12  2009/03/19 16:33:36  bcolombe
 * TOOL_POINT now derives from TOOL_GPS
 * Add ControlForm and v_OnGPSCoord
 *
 * Revision 1.11  2008/10/14 16:51:26  mju
 * Add v_generateRegion.
 *
 * Revision 1.10  2008/04/21 13:47:16  mju
 * Remove "doconly" method.
 *
 * Revision 1.9  2008/02/28 00:18:31  scowan
 * Added delegate.
 *
 * Revision 1.8  2008/02/15 23:37:32  scowan
 * Added manual entry stubs.
 *
 * Revision 1.7  2008/02/01 15:01:20  mju
 * Change button handlers to return bool.
 *
 * Revision 1.6  2008/01/04 18:59:22  mju
 * Generalize pointer button methods.
 *
 * Revision 1.5  2007/10/05 21:12:12  mju
 * Rename inclusion guards.
 *
 * Revision 1.4  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.3  2003/09/15 13:49:32  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/01/10 23:08:38  mju
 * Now works.
 *
 * Revision 1.1  2003/01/09 21:05:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLPOINT_H
#define	INC_MGUI_TOOLPOINT_H

#ifndef	INC_MGUI_TOOLGPS_H
#include <mgui/toolgps.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
struct POINTSTYLE;
namespace RVC {
	class STYLEPATTERN_SYMBOL;
	}
#endif


namespace MGUI {

//=====================================================================================================================

//! Single "point" tool.
class TOOL_POINT : public MGUI::TOOL_GPS {
	public:

		typedef fastdelegate::FastDelegate<bool(CHANGESEQ, DPOINT3D&)> DELEGATE_ONSETPOINT;
		
		//! Constructor.
		explicit TOOL_POINT (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_POINT ();

		//! Assignment from DPOINT2D.
		TOOL_POINT& operator= (
			const DPOINT2D& rhs
			);

		//! Assigment from DPOINT3D.
		TOOL_POINT& operator= (
			const DPOINT3D& rhs
			);

		//! Get current tool point coordinates.
		const DPOINT3D& GetPoint (
			) const { return (m_point); }

		//! Set delegate to call when a point's position is set or moved
		//! This allows the user of the tool to determine if the point is to be moved and where.
		//! Return 'true' for the delegate to modify the point.
		void SetDelegateOnSetPoint (
			DELEGATE_ONSETPOINT delegate
			) { m_DelegateOnSetPoint = delegate; }

		//! Set marker type and size.
		//! The default marker is MARKER_Plus with a size of 9 pixels.
		//! Will reset any previously set style.
		void SetMarker (
			MARKER marker,								//!< New marker type
			UINT16 size = 0							//!< New marker size, 0 to retain current size
			);

		//! Set style for using symbol.
		//! Will use marker unless either STYLEFLAG_UseSymbol or STYLEFLAG_PointSymbolBuiltIn is specified.
		void SetPointStyle (
			const POINTSTYLE& style,				//!< Style
			const RVC::STYLEPATTERN_SYMBOL *pSymbol = 0	//!< Optional symbol when STYLEFLAG_UseSymbol specified
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Enumeration constants
		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Outside,
			};
		// Tool settings
		MARKER m_marker;
		UINT16 m_markersize;
		POINTSTYLE *m_pStyle;
		RVC::STYLEPATTERN_SYMBOL *m_pSymbol;
		// Tool position values
		DPOINT3D m_point;						// Tool position
		// Values used during interaction
		ZONE m_CursorZone;						// Current cursor zone
		// Delegate for TOOL_POINT specific operations
		DELEGATE_ONSETPOINT m_DelegateOnSetPoint;
		
		// Manual Point Entry form forward declaration
		class MANUAL_POINT;
		friend class MANUAL_POINT;
		class FORM_POINT;
		FORM_POINT *m_pFormPoint;

		// Methods
		void ComputeCursorZone (const DPOINT2D&);

		// TOOL_BASE overrides.
		virtual MGUI::FORM* v_CreateControlForm (LAYOUT_PANE_BASE& ParentPane, CTRLOPT ctrlopt);
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);

		// TOOL_GPS overrides.
		virtual void v_OnGPSCoordinate (const DPOINT3D& point);

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLPOINT_H
