/**
 * \file mi32/balancedtree.h
 * \brief BALANCEDTREE template class definitions
 *
 * \if NODOC
 * $Id: balancedtree.h_v 1.1 2012/01/27 22:04:39 mju Exp $
 *
 * $Log: balancedtree.h_v $
 * Revision 1.1  2012/01/27 22:04:39  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_BALANCEDTREE_H
#define	INC_MI32_BALANCEDTREE_H

#ifndef INC_MI32_BTREEFUN_H
#include <mi32/btreefun.h>
#endif


#ifndef	INC_MI32_ERRCODES_H
#include <mi32/errcodes.h>
#endif

#ifndef	INC_MI32_ERRHANDLER_H
#include <mi32/errhandler.h>
#endif


//! Simple balanced tree container.
template <typename _ITEMTYPE> class BALANCEDTREE {
	public:

		//! Interface class to "dump" an item from the tree, used in TraverseTree().
		class DUMP {
			public:
				virtual ERRVALUE v_DumpItem (const _ITEMTYPE& item, INT32 TagNum) = 0;
			};
	
		//! Interface class to test two tree items, can use default if _T::operator<() defined
		class TEST {
			public:
				virtual ~TEST () {}
				virtual int v_TestItems (const _ITEMTYPE& p1, const _ITEMTYPE& p2) { return ((p1 < p2) ? -1 : ((p2 < p1) ? 1 : 0)); }
			};
	
	private:
 		#ifndef GENERATING_DOXYGEN_OUTPUT
		static int DumpFunc (void* p1, INT32* tagnum, void* ud) {
			return (static_cast<DUMP*>(ud)->v_DumpItem(*static_cast<_ITEMTYPE*>(p1), *tagnum));
			}
	
		static int TestFunc (const void* p1, const void* p2, void* ud) {
			return (static_cast<TEST*>(ud)->v_TestItems(*static_cast<const _ITEMTYPE*>(p1), *static_cast<const _ITEMTYPE*>(p2)));
			}
	   #endif // GENERATING_DOXYGEN_OUTPUT
	
	public:

		//! Constructor	
		BALANCEDTREE (
			) : m_BTreeHandle(0) {}
			
		//! Destructor	
		~BALANCEDTREE (
			) { 
			if (m_BTreeHandle != 0) {
				delete static_cast<TEST*>(BTreeGetUserData(m_BTreeHandle));
				BTreeFree(m_BTreeHandle); 
				}
			}

		//! Initialize the balanced tree giving an initial number of items
		ERRVALUE Allocate (
			INT32 InitialNumItems,
			TEST Test = TEST()
			) {
			m_BTreeHandle = BTreeAlloc(InitialNumItems, sizeof(_ITEMTYPE), TestFunc, new TEST(Test));
			if (m_BTreeHandle == 0) return (SetErrPosnC(EOutOfMemory));
			return (0);
			}
			
		//! Clear the balanced tree of all items
		ERRVALUE Clear (
			) { return (BTreeClear(m_BTreeHandle)); }
			
		//! Locate an item in the tree and return its key value.
		//! If the item does not exist, add it to the tree
		INT32 FindAndInsertKey (
			_ITEMTYPE& Item,
			INT32 CurTagNum,			//!< Tag number to use if 'Item' is added to the tree
			bool& AddedItem			//!< Set to 'true' if 'Item' is added to the tree
			) { 
			int add = 0;
			INT32 retval = BTreeFindKey(m_BTreeHandle, &Item, CurTagNum, &add);
			AddedItem = (add != 0);
			return (retval);
			}

		//! Determine if the balanced tree Allocate() method has been called
		bool IsAllocated (
			) const { return (m_BTreeHandle != 0); }
			
		//! Locate an item in the tree and return its key value.
		//! @return -1 = Not found, < -1 Error, else the tag number associated with the data.
		INT32 LocateKey (
			const _ITEMTYPE& Item
			) { return (BTreeSearchKey(m_BTreeHandle, const_cast<_ITEMTYPE*>(&Item))); } 
			
		//! Traverse the balanced tree in sorted order.
		ERRVALUE TraverseTree (
			DUMP& Dump
			) { return (BTreeDumpT(m_BTreeHandle, DumpFunc, &Dump)); }
			
	private:
 		#ifndef GENERATING_DOXYGEN_OUTPUT
		void *m_BTreeHandle;	

		BALANCEDTREE (const BALANCEDTREE& rhs);
		BALANCEDTREE& operator=(const BALANCEDTREE& rhs);
	   #endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_BALANCEDTREE_H
