/**
 * \file longlist.h <mi32/longlist.h>
 * \brief Definitions for LONGLIST
 *
 * \if NODOC
 * $Id: longlist.h_v 1.11 2011/10/19 17:43:40 mju Exp $
 *
 * $Log: longlist.h_v $
 * Revision 1.11  2011/10/19 17:43:40  mju
 * Remove redundant hdr.
 *
 * Revision 1.10  2010/08/19 17:39:48  mju
 * Use unsignedset in setBits.
 *
 * Revision 1.8  2006/10/03 15:59:38  dwilliss
 * Added a method to get the list as simple array of int32
 *
 * Revision 1.7  2005/08/24 17:27:14  dwilliss
 * Changed to a real class
 *
 * Revision 1.1  1999/04/08  14:44:29  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_LONGLIST_H
#define	INC_MI32_LONGLIST_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class UNSIGNEDSET;
#endif


//! Similar to a SIMPLE_ARRAY<UINT32>, but with some differences.
//! A LONGLIST serves two purposes. 
//! 	When possible, it keeps track of a list of numbers using a BITSET.
//!	It can also be made to keep track of a set of numbers like a SIMPLE_ARRAY
//!	but with an extra UINT8 flags for each entry.
struct LONGLIST {
	LONGLIST ();
	~LONGLIST ();

	//! Clear the list, but does not free memory.
	void Clear ();

	void Free ();
	ERRVALUE Add (INT32 value);
	ERRVALUE AddUnique (INT32 value);
	ERRVALUE SetBits (const UNSIGNEDSET& set);
	ERRVALUE AddWithFlag (INT32 value, UINT8 flag);
	ERRVALUE AddUniqueWithFlag (INT32 value, UINT8 flag);
	ERRVALUE Remove (INT32 value);
	bool CheckIn (INT32 value) const;
	bool GetNext (INT32 *ip, INT32*valp) const;
	bool GetFirst (INT32 *ip, INT32*valp) const;
	bool GetLast (INT32*valp) const;
	INT32 GetList (INT32** listp) const;
	ERRVALUE GetList (SIMPLE_ARRAY<INT32>& List) const;
	UINT8 GetFlags (INT32 Index) const { return m_flags[Index]; }
	INT32 GetValue (INT32 Index) const { return m_List[Index]; }

	bool HasFlags (
		) const { return (!m_UseBits && m_flags != 0); }

	bool IsSorted (
		) const { return m_sorted; }

	void SetUseBits (
		bool val = true
		) { m_UseBits = val; }

	void SetUserData (
		INT32 val
		) { m_UserData = val; }

	INT32 GetNumItems (
		) const { return m_num; }

	INT32 GetUserData (
		) const { return m_UserData; }

	ERRVALUE Sort (
		int (*cmpfunc)(INT32, INT32, void*) = 0,
		void* cmpdata = 0
		);

	//! Set the sorted flag without actually sorting.
	//! This is useful if you know you added the items in sorted order.
	//! You may also want to set it to false to force a re-sort if you change
	//! the sort criteria.
	void SetSorted (
		bool val = true
		) {
		m_sorted = val;
		}

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT

	INT32 m_NumAlloc;
	INT32 m_num;
	INT32 *m_List;								// Shouldn't access directly 
	UINT8 *m_flags; 
	bool m_sorted;
	bool m_UseBits;
	INT32 m_UserData;							// Can be used for anything you want 

	static int HeapSortCompare(INT32, INT32, void*);
	static int HeapSortSwap(INT32, INT32, void*);

	#endif
	};

#endif	// INC_MI32_LONGLIST_H
