/**
 * \file mi32/math.h
 * \brief Math functions and prototypes, also includes standard C math hdr.
 *
 * \if NODOC
 * $Id: math.h_v 1.18 2012/10/09 14:09:59 mju Exp $
 *
 * $Log: math.h_v $
 * Revision 1.18  2012/10/09 14:09:59  mju
 * *** empty log message ***
 *
 * Revision 1.17  2012/10/09 13:43:10  mju
 * Add roundmode enum.
 *
 * Revision 1.16  2012/01/26 17:45:55  mju
 * Fix inline log2i parms.
 *
 * Revision 1.15  2012/01/26 17:13:49  mju
 * Forgot inline on log2i.
 *
 * Revision 1.14  2012/01/26 15:29:02  mju
 * Change log2i to return int and add signed variants.
 *
 * Revision 1.13  2012/01/26 15:22:33  mju
 * Add log2i fns.
 *
 * Revision 1.12  2011/07/12 16:06:58  mju
 * Add isInfNeg.
 *
 * Revision 1.11  2011/05/19 17:37:25  mju
 * Change fast_round_nolimit to be like exact for non-win32.
 *
 * Revision 1.10  2011/05/19 17:33:05  mju
 * Add fast_round_exact.
 *
 * Revision 1.9  2008/11/12 22:29:12  mju
 * Another doxygen fix.
 *
 * Revision 1.8  2008/11/11 18:42:26  mju
 * Fix doxygen file tag.
 *
 * Revision 1.7  2007/06/29 20:19:07  mju
 * Add fast_ceil/floor/truncate_nearint functions.
 *
 * Revision 1.6  2007/05/04 13:11:12  mju
 * Remove old round macro.
 *
 * Revision 1.5  2007/05/03 21:26:51  mju
 * Add fast_round_limited/nolimit/positive variants.
 * Make fast_round do fast_round_limited and deprecate.
 * Change fast_truncate to simple truncation as now works on win32.
 *
 * Revision 1.4  2006/07/27 14:53:10  scowan
 * Include cmath.
 *
 * Revision 1.3  2006/03/10 15:48:12  mju
 * Not allowed asm on win64.
 *
 * Revision 1.2  2005/09/08 21:25:03  mju
 * Don't declare legacy round macro if no_deprecated.
 *
 * Revision 1.1  2005/07/15 16:24:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MATH_H
#define	INC_MI32_MATH_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_MATH_H
#ifndef GENERATING_DOXYGEN_OUTPUT
#include <math.h>
#define	INC_MATH_H
#endif
#endif

#ifndef	INC_CMATH
#include <cmath>		// Definitions for std::abs() for floating point types.
#define	INC_CMATH
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif


//!\addtogroup math Math Functions
//!@{

#if defined(__cplusplus)
extern "C" {
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT

#if defined(WIN32)
	#if __STDC__ && _MSC_VER >= 1200
	#define hypot		_hypot
	#endif

#else
	double hypot (double, double);			// Define this for everywhere else except Win32.  In Win32 it is a DLL import
#endif

#endif	// GENERATING_DOXYGEN_OUTPUT

//! Returns the IEEE representation of Not A Number.
LIBEXPORT double IEEE_NaN (void);

//! Returns the IEEE representation of Infinity.
LIBEXPORT double IEEE_Infinity (void);

#if defined(__cplusplus)
}
#endif

//---------------------------------------------------------------------------------------------------------------------

//! Rounding mode.
enum ROUNDMODE {
	ROUNDMODE_None =			0,		//!< Don't round
	ROUNDMODE_Nearest =		1,		//!< Round to nearest value, .5 will round higher
	ROUNDMODE_Down =			2,		//!< Round to lower value
	ROUNDMODE_Up =				3,		//!< Round to higher value
	ROUNDMODE_TowardZero =	4,		//!< Round toward 0
	};

//---------------------------------------------------------------------------------------------------------------------

#define	NEARINTADJ	.0000001

// On Intel, avoid slow function calls.
// Accounts for Intel rounding .5 values to nearest -even- integer, corrects for up to +/- 1000000


//! Fast return of smallest integer greater than or equal to specified value.
//! This function is at least 5X faster than using static_cast<INT32>(ceil(x)) on Intel.
//! Accuracy limitations on Windows 32-bit:
//! Values in the range from -1000000 to +1000000 at exactly .0 will be handled correctly.
//! Values outside that range may not always be correct.  In addition, values in the
//! range of -1000 to 1000 which are within 1E-10 of .0 may not be correctly rounded.
//! The FAST_... functions should not be used if exact accuracy is required.
inline INT32 FAST_CEIL (double x) {
#if defined(WIN32) && defined(__cplusplus) && !defined(WIN64)
	static const double d = .4999999999;
	INT32 result;
	__asm fld x
	__asm fadd d
	__asm fistp result
	return result;
#else
	return (static_cast<INT32>(ceil(x)));
#endif
	}


//! Fast return of smallest integer greater than or equal to specified value, with near-integer adjustment.
//! This function is at least 5X faster than using static_cast<INT32>(ceil(x)) on Intel.
//! Accuracy limitations on Windows 32-bit:
//! Values in the range from -1000000 to +1000000 at exactly .0 will be handled correctly.
//! Values outside that range may not always be correct.  In addition, values in the
//! range of -1000 to 1000 which are within 1E-10 of .0 may not be correctly rounded.
//! The FAST_... functions should not be used if exact accuracy is required.
inline INT32 FAST_CEIL_NEARINT (double x) {
#if defined(WIN32) && defined(__cplusplus) && !defined(WIN64)
	static const double d = .4999998;
	INT32 result;
	__asm fld x
	__asm fadd d
	__asm fistp result
	return result;
#else
	return (static_cast<INT32>(ceil(x-NEARINTADJ)));
#endif
	}


//! Fast return of largest integer less than or equal to specified value.
//! This function is at least 5X faster than using static_cast<INT32>(floor(x)) on Intel.
//! Accuracy limitations:
//! Accuracy limitations on Windows 32-bit:
//! Values outside that range may not always be correct.  In addition, values in the
//! range of -1000 to 1000 which are within 1E-10 of .0 may not be correctly rounded.
//! The FAST_... functions should not be used if exact accuracy is required.
inline INT32 FAST_FLOOR (double x) {
#if defined(WIN32) && defined(__cplusplus) && !defined(WIN64)
	static const double d = -.4999999999;
	INT32 result;
	__asm fld x
	__asm fadd d
	__asm fistp result
	return result;
#else
	return (static_cast<INT32>(floor(x)));
#endif
	}


//! Fast return of largest integer less than or equal to specified value, with near-integer adjustment.
//! This function is at least 5X faster than using static_cast<INT32>(floor(x)) on Intel.
//! Accuracy limitations:
//! Accuracy limitations on Windows 32-bit:
//! Values outside that range may not always be correct.  In addition, values in the
//! range of -1000 to 1000 which are within 1E-10 of .0 may not be correctly rounded.
//! The FAST_... functions should not be used if exact accuracy is required.
inline INT32 FAST_FLOOR_NEARINT (double x) {
#if defined(WIN32) && defined(__cplusplus) && !defined(WIN64)
	static const double d = -.4999998;
	INT32 result;
	__asm fld x
	__asm fadd d
	__asm fistp result
	return result;
#else
	return (static_cast<INT32>(floor(x+NEARINTADJ)));
#endif
	}

//! Fast round of value to nearest integer, with X.5 values rounded up.
//! This function is typically faster than using static_cast<INT32>(floor(x+.5)).
//! No known accuracy limitations, assuming value is in valid INT32 range.
inline INT32 FAST_ROUND_EXACT (double x) {
	if (x < 0) {
		return (-static_cast<INT32>(-x+.5));
		}
	else {
		return (static_cast<INT32>(x+.5));
		}
	}

//! Fast round of value to nearest integer, with X.5 values rounded up.
//! This function is approximately 3X faster than using static_cast<INT32>(floor(x+.5)) on Win32.
//! Accuracy limitations for win32:
//! Values in the range from -1000000 to +1000000 at exactly .5 will be handled correctly.
//! Values outside that range may not always be rounded up.  In addition, values in the
//! range of -1000 to 1000 which are within 1E-10 of .5 may not be correctly rounded down.
//! No known accuracy limitations for non-win32:
//! Use FAST_ROUND_EXACT if exact accuracy is required.
inline INT32 FAST_ROUND_NOLIMIT (double x) {
	#if defined(WIN32) && defined(__cplusplus) && !defined(WIN64)
		static const double d = 1E-10;
		INT32 result;
		__asm fld x
		__asm fadd d
		__asm fistp result
		return result;
	#else
		if (x < 0) {
			return (-static_cast<INT32>(-x+.5));
			}
		else {
			return (static_cast<INT32>(x+.5));
			}
	#endif
	}

//! Fast round (within limits) of value to nearest integer, with X.5 values rounded up.
//! Values less than -1000000 at exactly X.5 will be incorrectly.
//! Values > 2 billion may be truncated.
//! If values are known to always be positive, the FAST_ROUND_POSITIVE function will be faster.
inline INT32 FAST_ROUND_LIMITED (double x) {
	return (static_cast<INT32>(x+1000000.5)-1000000);
	}

//! Fast round of positive value to nearest integer, with X.5 values rounded up.
//! Negative values will not be rounded correctly, this function should only be used for positive values.
inline INT32 FAST_ROUND_POSITIVE (double x) {
	return (static_cast<INT32>(x+.5));
	}

//! Fast truncation of value toward zero.
//! No known accuracy limitations.
inline INT32 FAST_TRUNCATE (double x) {
	return (static_cast<INT32>(x));
	}

//! Fast truncation of value toward zero, with near-integer adjustment.
inline INT32 FAST_TRUNCATE_NEARINT (double x) {
	return (static_cast<INT32>(x+NEARINTADJ));
	}


#undef	NEARINTADJ

// deprecated Replaced by FAST_ROUND_EXACT/NOLIMIT/LIMITED/POSITIVE
#define	FAST_ROUND		FAST_ROUND_EXACT
#ifdef WIN32
#pragma deprecated("FAST_ROUND")
#endif


//---------------------------------------------------------------------------------------------------------------------
//		Macros for determine special floating-point values					   

/*
 * Determination of IEEE NaN:  On most systems, the high word is
 * 0x7FF8 or 0xFFF8 (high bit is sign)
 */

#if defined(BYTEORDER_HiLo)
	#if defined(__cplusplus)
		inline bool IsNaN (double x) {return ((reinterpret_cast<UINT16*>(&x)[0] & 0x7FF8) == 0x7FF8);}
	#else 
		#define IsNaN(x) ( ( ( (UINT16*)&(x) )[0] & 0x7FF8) == 0x7FF8)
	#endif
#else
	#if defined(__cplusplus)
		inline bool IsNaN (const double& x) {return ((reinterpret_cast<const UINT16*>(&x)[3] & 0x7FF8) == 0x7FF8);}
	#else 
		#define IsNaN(x) ( ( ( (UINT16*)&(x) )[3] & 0x7FF8) == 0x7FF8)
	#endif
#endif


#ifdef BYTEORDER_HiLo
	#if defined(__cplusplus)
		inline bool IsInfPos (double x) {return ((reinterpret_cast<UINT16*>(&x)[0] & 0x7FFF) == 0x7FF0);}
	#else 
		#define IsInfPos(x) ( ( ( (UINT16*)&(x) )[0] & 0x7FFF) == 0x7FF0)
	#endif
#else
	#if defined(__cplusplus)
		inline bool IsInfPos (double x) {return ((reinterpret_cast<UINT16*>(&x)[3] & 0x7FFF) == 0x7FF0);}
	#else 
		#define IsInfPos(x) ( ( ( (UINT16*)&(x) )[3] & 0x7FFF) == 0x7FF0)
	#endif
#endif

#ifdef BYTEORDER_HiLo
	#if defined(__cplusplus)
		inline bool IsInfNeg (double x) {return ((reinterpret_cast<UINT16*>(&x)[0] & 0xFFFF) == 0xFFF0);}
	#else 
		#define IsInfNeg(x) ( ( ( (UINT16*)&(x) )[0] & 0xFFFF) == 0xFFF0)
	#endif
#else
	#if defined(__cplusplus)
		inline bool IsInfNeg (double x) {return ((reinterpret_cast<UINT16*>(&x)[3] & 0xFFFF) == 0xFFF0);}
	#else 
		#define IsInfNeg(x) ( ( ( (UINT16*)&(x) )[3] & 0xFFFF) == 0xFFF0)
	#endif
#endif


#ifndef	NO_DEPRECATED

#ifndef NAN
// If already defined, assume it's a better defn
#define NAN IEEE_NaN()
#endif

#define IND IEEE_NaN()
#define INF IEEE_Infinity()
#define NINF (-IEEE_Infinity())

#ifdef WIN32
#pragma deprecated("NAN")
#pragma deprecated("IND")
#pragma deprecated("INF")
#pragma deprecated("NINF")
#endif

#endif	// NO_DEPRECATED


//! Convert IEEE floating point to VAX floating point.
void ConvIEEEToVAX (
	double ieee_value,	 		//!< IEEE floating point value
	UINT8 *vax_value			//!< Pointer to 8 byte UINT8 array for VAX floating point value
	);

//! Convert VAX floating point to IEEE floating point.
//!
//! @return: IEEE floating point value
double ConvVAXToIEEE (
	const UINT8 *vax_value			//!< Pointer to 8 byte UINT8 array for VAX floating point value
	);

//! Compute base-2 logarithm of UINT32 value.
//! @return base-2 logarithm (0-32) or -1 if input value is 0.
int log2i (
	UINT32 value,						//!< Value to compute for
	bool roundup = true				//!< Whether to round result up (true) or down (false)
	);

//! Compute base-2 logarithm of INT32 value.
//! @return base-2 logarithm (0-31) or -1 if input value <= 0.
inline int log2i (
	INT32 value,						//!< Value to compute for
	bool roundup = true				//!< Whether to round result up (true) or down (false)
	) { return ((value > 0) ? log2i(static_cast<UINT32>(value),roundup) : -1); }

//! Compute base-2 logarithm of UINT64 value.
//! @return base-2 logarithm (0-64) or -1 if input value is 0.
int log2i (
	UINT64 value,						//!< Value to compute for
	bool roundup = true				//!< Whether to round result up (true) or down (false)
	);

//! Compute base-2 logarithm of INT64 value.
//! @return base-2 logarithm (0-63) or -1 if input value <= 0.
inline int log2i (
	INT64 value,						//!< Value to compute for
	bool roundup = true				//!< Whether to round result up (true) or down (false)
	) { return ((value > 0) ? log2i(static_cast<UINT64>(value),roundup) : -1); }

//!@}

#undef LIBEXPORT

#endif	// INC_MI32_MATH_H
