/**
 * \file mi32/miglx.h
 * \brief MIGLX class definitions
 *
 * \if NODOC
 * $Id: miglx.h_v 1.6 2012/03/23 16:55:34 mju Exp $
 *
 * $Log: miglx.h_v $
 * Revision 1.6  2012/03/23 16:55:34  mju
 * Add optional rectangle for copyFromMemory.
 *
 * Revision 1.5  2012/03/22 20:52:49  mju
 * Add copyBackToFront.
 *
 * Revision 1.4  2012/03/09 21:24:08  mju
 * Add checkError.
 *
 * Revision 1.3  2012/03/07 21:17:26  mju
 * Add copyFromMemory.
 *
 * Revision 1.2  2012/03/02 16:09:50  mju
 * Add context nested class.
 *
 * Revision 1.1  2012/03/01 16:39:07  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MIGLX_H
#define	INC_MI32_MIGLX_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifndef	_XUTIL_H_
#include <X11/Xutil.h>
#endif


#ifdef WIN32
#define	GLX_IS_ALLOWED
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
typedef struct __GLXcontextRec *GLXContext;
namespace MGD {
	class DEVICE_MEM32;
	}
#endif


//! Helper class for X11 OpenGL Extension (GLX).
class MIGLX {
	public:

		//! Buffer(s) to select for drawing.
		enum BUFFER {
			BUFFER_Front =				0x01,			//!< Front (on-screen) buffer only, both left and right if available
			BUFFER_Back =				0x02,			//!< Back (off-screen) buffer only, both left and right if available
			BUFFER_Left =				0x10,			//!< Left buffer only, both front and back if available
			BUFFER_Right =				0x20,			//!< Right buffer only, both front and back if available
			BUFFER_FrontLeft =		0x11,			//!< Front left buffer only
			BUFFER_FrontRight =		0x21,			//!< Front right buffer only
			BUFFER_BackLeft =			0x12,			//!< Back left buffer only
			BUFFER_BackRight =		0x22,			//!< Back right buffer only
			BUFFER_All =				0x00,			//!< All buffers concurrently
			};

		enum MODE {
			MODE_Any =				0x00,
			MODE_DoubleBuffer =	0x01,
			MODE_Stereo =			0x02,
			};

		class CONTEXT {
			public:

				//! Constructor.
				CONTEXT ();

				//! Destructor.
				~CONTEXT ();

				//! Copy pixels from back to front buffers.
				//! Has no effect if not double-buffered.
				//! Will copy both left and right buffers if stereo.
				void CopyBackToFront (
					const LRECT2D *pRect = 0				//!< Optional rectangle MGD coordinates (0,0 at upper left, inverted from OpenGL)
					);

				//! Copy into context buffer(s) from memory.
				//! Must make context "current" before using.
				ERRVALUE CopyFromMemory (
					MGD::DEVICE_MEM32 *pDeviceSrc,		//!< Device to copy from
					BUFFER buffer,								//!< Which buffer(s) to copy into/from
					const LRECT2D *pRect = 0				//!< Optional rectangle MGD coordinates (0,0 at upper left, inverted from OpenGL)
					);

				//! Create context with specified mode if available.
				//! Does not make context "current" for subsequent OpenGL functions.
				//! @return true if successful, false if mode not supported.
				bool Create (
					MODE mode				//!< Required mode
					);

				//! Destroy context.
				void Destroy ();

				//! Determine if has been created.
				bool IsCreated () const
					{ return (m_GlxContext != 0); }

				//! Determine if context is double-buffered.
				bool IsDoubleBuffered () const
					{ return ((m_mode & MODE_DoubleBuffer) != 0); }

				//! Determine if context uses stereo mode.
				bool IsStereo () const
					{ return ((m_mode & MODE_Stereo) != 0); }

				//! Make context "current" for subsequent OpenGL functions.
				void MakeCurrent (
					XID xwindow
					);

				//! Set model/projection transformations to identity such that model coordinates are window coordinates (at z = 0).
				void SetTransIdentity (
					unsigned int winwidth = 0,			//!< Window width if known, in pixels
					unsigned int winheight = 0			//!< Window height if known, in pixels
					);

				//! Swap buffers if double-buffering used.
				void SwapBuffers ();

				//! Translate from MIGLX::BUFFER to OpenGL buffer enum value.
				int TranslateBuffer (
					BUFFER buffer
					) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				GLXContext m_GlxContext;
				XID m_xwindow;
				MODE m_mode;

				bool GetWindowDimensions (unsigned int& width, unsigned int& height) const;

				CONTEXT (const CONTEXT&);
				CONTEXT& operator= (const CONTEXT&);
				#endif
			};

		//! Check for OpenGL error.
		//! Will build message string with all available errors.
		static ERRVALUE CheckError ();

		//! Determine if specific mode is available.
		//! Will return false if GLX extension is not enabled.
		static bool HasMode (
			MODE mode						//!< Combination of desired MODE values
			);

		//! Determine if GLX extension is enabled.
		static bool IsEnabled ();

	private:			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		static XVisualInfo* ChooseVisual (MODE mode);
		#endif
	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MIGLX::MODE)
#endif

#endif	// INC_MI32_MIGLX_H
