/**
 * \file readwritelock.h <mi32/readwritelock.h>
 * \brief Definition for Read / Write Syncronization locks
 *
 * \if NODOC
 * $Id: readwritelock.h_v 1.6 2007/08/17 17:05:28 scowan Exp $
 *
 * $Log: readwritelock.h_v $
 * Revision 1.6  2007/08/17 17:05:28  scowan
 * Attempt to get doxygen to parse properly.
 *
 * Revision 1.5  2007/07/19 15:35:29  scowan
 * Added more methods for read and write lock.
 *
 * Revision 1.4  2007/07/18 21:45:09  scowan
 * Finished implementation for all platforms.
 *
 * Revision 1.3  2007/07/13 21:21:06  scowan
 * Make sure to export read and write lock nested classes.
 *
 * Revision 1.2  2007/07/13 21:20:20  scowan
 * In the middle of updating for 7.4.
 *
 * Revision 1.1  2005/09/21 22:31:13  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_READWRITELOCK_H
#define	INC_MI32_READWRITELOCK_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace MITHREAD {

//! Read/Writer lock class that implements the read/write lock paradigm.
//! A thread can use the nested classes READLOCK and WRITELOCK to obtain the lock held by READWRITESYNC
class CLASSLIBEXPORT READWRITESYNC {
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	public:
		//! Read Lock class to obtain and hold a read lock on the READWRITESYNC object until dtor is called
		//! Multiple threads can obtain read locks at the same time
		class CLASSLIBEXPORT READLOCK {
			public:
				//! Default ctor
		   	READLOCK (
					);

				//! Pre-init ctor
		   	READLOCK (
					READWRITESYNC& rwsync, 			//!< Synchronization read/write object to obtain a read lock on
					bool bInitialLock = true,		//!< Get a lock on ctor call, false to obtain one later
					bool bTryLock = false			//!< Use try lock semantics, call IsLocked() to determine if lock is granted, recursion support must be enabled
					);

				//! Default dtor, will release the lock if IsLocked() == true
				~READLOCK (
					);

				//! Determine that a lock was obtained, only useful if ctor is passed 'true' for bTryLock parameter
				bool IsLocked (
					) const { return (m_bIsLocked); }

				//! Obtains a lock on READWRITESYNC object passed in the ctor.  Only useful if 'false' is passed in bInitialLock parameter.
				void Lock (
					);

				//! Obtains a lock on READWRITESYNC object passed in.
				void Lock (
					READWRITESYNC& rwsync 			//!< Synchronization read/write object to obtain a read lock on
					);

				//! Attempts to obtain a lock on READWRITESYNC object passed in the ctor.  
				//! Only useful if 'false' is passed in bInitialLock parameter and recursion support is enabled.
				//! @return 'True' if the lock was obtained, 'false' if not.
				bool TryLock (
					);

				//! Attempts to obtain a lock on READWRITESYNC object passed in the ctor.  
				//! Only useful if recursion support is enabled.
				//! @return 'True' if the lock was obtained, 'false' if not.
				bool TryLock (
					READWRITESYNC& rwsync 			//!< Synchronization read/write object to obtain a read lock on
					);

				//! Releases the lock on READWRITESYNC object.
				void Unlock (
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				PRIV* m_pPriv;
				bool m_bIsLocked;

				READLOCK (const READLOCK& rhs);
				READLOCK& operator=(const READLOCK& rhs);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};
			

		//! Write Lock class to obtain and hold a write lock on the READWRITESYNC object until dtor is called
		//! Only one thread can obtain a write lock at a time.
		class CLASSLIBEXPORT WRITELOCK {
			public:
				//! Default ctor
		   	WRITELOCK (
					);

				//! Pre-init ctor
				//! Default ctor
		   	WRITELOCK (
					READWRITESYNC& rwsync, 			//!< Synchronization read/write object to obtain a write lock on
					bool bInitialLock = true,		//!< Get a lock on ctor call, false to obtain one later
					bool bTryLock = false			//!< Use try lock semantics, call IsLocked() to determine if lock is granted, recursion support must be enabled
					);

				//! Default dtor, will release the lock if IsLocked() == true
				~WRITELOCK (
					);

				//! Determine that a lock was obtained, only useful if ctor is passed 'true' for bTryLock parameter
				bool IsLocked (
					) const { return (m_bIsLocked); }

				//! Obtains a lock on READWRITESYNC object passed in the ctor.  Only useful if 'false' is passed in bInitialLock parameter.
				void Lock (
					);

				//! Obtains a lock on READWRITESYNC object passed in.
				void Lock (
					READWRITESYNC& rwsync 			//!< Synchronization read/write object to obtain a read lock on
					);

				//! Attempts to obtain a lock on READWRITESYNC object passed in the ctor.
				//! Only useful if 'false' is passed in bInitialLock parameter and recursion support is enabled.
				//! @return 'True' if the lock was obtained, 'false' if not.
				bool TryLock (
					);

				//! Attempts to obtain a lock on READWRITESYNC object passed in.
				//! Only useful if recursion support is enabled.
				//! @return 'True' if the lock was obtained, 'false' if not.
				bool TryLock (
					READWRITESYNC& rwsync 			//!< Synchronization read/write object to obtain a read lock on
					);

				//! Releases the lock on READWRITESYNC object.
				void Unlock (
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				PRIV* m_pPriv;
				bool m_bIsLocked;

				WRITELOCK (const WRITELOCK& rhs);
				WRITELOCK& operator=(const WRITELOCK& rhs);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Reader/Writer Lock ctor
		READWRITESYNC (
			bool bSupportRecursion = false		//!< If true, allow threads to obtain this lock while it already has this lock
															//!< Set to 'true' for try locking semantics to work correctly
															//!< It is far more efficient to not use recursion support
			);

		//! Reader/Writer Lock dtor
		~READWRITESYNC (
		 	);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PRIV* m_pPriv;

		READWRITESYNC (const READWRITESYNC& rhs);
		READWRITESYNC& operator=(const READWRITESYNC& rhs);
		
		friend class READLOCK;
		friend class WRITELOCK;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of MITHREAD namespace

#undef	CLASSLIBEXPORT
#endif	//!< INC_MI32_READWRITELOCK_H
