/***
 * \file rvc/opacitymask.h 
 * \brief interface for the RVC::OPACITYMASK class.
 *
 *	\if NODOC
 *	$Id: opacitymask.h_v 1.17 2011/12/21 22:23:24 mju Exp $
 *
 *	$Log: opacitymask.h_v $
 *	Revision 1.17  2011/12/21 22:23:24  mju
 *	Remove redundant hdrs.
 *
 *	Revision 1.16  2008/01/22 23:30:42  scowan
 *	Support optional object filter for opacity masks.
 *
 *	Revision 1.15  2007/01/19 16:10:27  mju
 *	Use color_array.
 *
 *	Revision 1.14  2006/10/27 22:50:18  scowan
 *	Made getCellType virtual.
 *
 *	Revision 1.13  2006/10/19 20:40:23  mju
 *	Use pyramid_default instead of pyramid_none.
 *
 *	Revision 1.11  2006/06/27 15:04:40  scowan
 *	Make method now fills in objitem with created object info.
 *
 *	Revision 1.10  2006/05/31 15:06:09  scowan
 *	Added setCacheRect virtual method.
 *
 *	Revision 1.9  2006/04/14 21:15:01  mju
 *	Add makeparm setting for initial value.
 *
 *	Revision 1.1  2004/02/24 18:36:22  scowan
 *	Initial revision
 *	\endif
***/

#ifndef INC_RVC_OPACITYMASK_H
#define INC_RVC_OPACITYMASK_H

#ifndef INC_RVC_NULLMASK_H
	#include <rvc/nullmask.h>
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_Raster RVC::RASTER::USAGE_Opacity child 1-bit, 4-bit or 8-bit raster objects.
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Raster (RVC::RASTER::USAGE_Opacity).
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_Hyperspectral
//!		- RVC::OBJTYPE_RasterSet
//!		- RVC::OBJTYPE_Thumbnail
class OPACITYMASK : public RVC::IMAGE {
	public:
		
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class GETOBJFILTER_ISVALIDOPACITYMASK;
		class MAKEPARMS;
		class OBJECT_FILTER;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Default constructor.
		OPACITYMASK (
			);

		//! Copy constructor.
		OPACITYMASK (
			const RVC::OPACITYMASK& rhs
			);

		//! Destructor
		virtual ~OPACITYMASK (
			);

		//! Assignment.
		RVC::OPACITYMASK& operator= (
			const RVC::OPACITYMASK& rhs
			);

		//! Obtain number of columns in a opacity mask tile
		//! @return number of columns in the opacity mask tile
		INT32 GetNumTileColumns (
			) const;

		//! Obtain number of lines in a opacity mask tile
		//! @return number of lines in the opacity mask tile
		INT32 GetNumTileLines (
			) const;

		//! Obtain number of tiles in a opacity mask row
		//! @return number of row tiles
		INT32 GetNumXTiles (
			) const;

		//! Obtain number of tiles in a opacity mask column
		//! @return number of column tiles
		INT32 GetNumYTiles (
			) const;

		//! Return list of available raster pyramid sub-rasters.
		//!	The sample rate list will be sorted in ascending order and the array and list are aligned.
		ERRVALUE GetPyramidList (
			SIMPLE_ARRAY<INT32>& SampleList,			//!< Simple array of available sample rates RETURNED
			RVC::OBJITEMLIST& PyramidList				//!< List of pyramid raster NullMask ObjItem's RETURNED
			);
			
		//! Obtain number of bytes in a opacity mask tile
		//! @return size of opacity mask tile in bytes
		INT32 GetTileSizeBytes (
			) const;

		//! Determine if a mask is present or the image has opacity values defined
		//! @return 'True' if meaningful opacity values or mask exists, 'false' otherwise
		bool HasOpacityValues (
			) const;

		//! Create an opacity mask object under the RVC::IMAGE parent.
		//! CellType == RVC::IMAGE::CELLTYPE_Binary, RVC::IMAGE::CELLTYPE_Unsigned_4 or RVC::IMAGE::CELLTYPE_Unsigned_8
		//! Tile Size == Same as parent
		//! Scale == Same as parent
		ERRVALUE Make (
	    	const RVC::IMAGE& parent,					//!< Parent object to create opacity mask under
			const RVC::DESCRIPTOR& Descriptor,
			const MAKEPARMS& MakeParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create an opacity mask object under the RVC::IMAGE parent.
		//! CellType == RVC::IMAGE::CELLTYPE_Binary, RVC::IMAGE::CELLTYPE_Unsigned_4 or RVC::IMAGE::CELLTYPE_Unsigned_8
		//! Compression type == RVC::RASTER::COMPTYPE_RLE or RVC::RASTER::COMPTYPE_DCPM
		//! Tile Size == Same as parent
		//! Scale == Same as parent
		ERRVALUE Make (
	    	RVC::OBJITEM& OpacityItem,					//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
			const char* Source = 0,						//!< Source of opacity mask creation, default is GetAppName()
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create pyramid opacity mask set under the existing raster pyramids
		ERRVALUE MakePyramidSet (
			IMAGE::PYRAMID flags = IMAGE::PYRAMID_Default,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Open the last used opacity mask under the RVC::IMAGE parent.
		//! If an opacity mask cannot be found, it will use the opacity value from the parent 
		//! and treat it like an opacity mask
		ERRVALUE Open (
	    	const RVC::IMAGE& parent,					//!< Parent image object to open opacity mask under
			OBJECT::OPENMODE OpenMode = OBJECT::OPENMODE_Read,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);			

		//! Open an opacity mask specified by 'OpacityMaskItem'
		ERRVALUE Open (
	    	const RVC::IMAGE& parent,							//!< Parent image object
			const RVC::OBJITEM& OpacityMaskItem,			//!< Specfic opacity mask to open
			OBJECT::OPENMODE OpenMode = OBJECT::OPENMODE_Read,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);			

		//! Read a line of opacity mask cells up to a certain length
		//! If there is more than one opacity component (null mask, colormap, opacity mask)
		//!    the order of opacity mask combinations are as follows 1. Null mask, 2. Colormap, 3. Opacity mask / Alpha band
		//! @return '>0' if there can exist opacity values other than 255, '0' if all values are known to be == 255, '<0' error
		int Read (
			INT32 Line,										//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			UINT8 *data,									//!< Buffer to read into
			INT32 Length = 1								//!< Number of columns to read
			) const;

		//! Read a column of opacity mask cells up to a certain length
		//! If there is more than one opacity component (null mask, colormap, opacity mask)
		//!    the order of opacity mask combinations are as follows 1. Null mask, 2. Colormap, 3. Opacity mask / Alpha band
		//! @return '>0' if there can exist opacity values other than 255, '0' if all values are known to be == 255, '<0' error
		int ReadColumn (
			INT32 StartLine,								//!< Raster line to start reading on
			INT32 Column,									//!< Raster column to read
			UINT8 *data,									//!< Buffer to read into
			INT32 Length = 1								//!< Number of lines to read
			) const;
			
		// Set the color map to use to obtain opacity value from the color map
		ERRVALUE SetColorMap (
			const COLOR_ARRAY& ColorMap
			);

		//! Write a line of opacity mask cells up to a certain length.
		//! If the opacity values are from the parent image, a OpacityMask raster will
		//! be created under the parent image with a celltype of IMAGE::CELLTYPE_Unsigned_8, the
		//! current opacity values setup in the new object, then the buffer will be written
		ERRVALUE Write (
			INT32 Line,										//!< Raster line to write
			INT32 StartColumn,							//!< Raster column to start writing on
			const UINT8* data,									//!< Buffer to write from
			INT32 Length = 1								//!< Number of columns to write
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		IMAGE* m_Image;			//!< Open image object
		RASTER m_OpacityObj;		//!< Open raster opacity mask object
		NULLMASK m_NullObj;			//!< Open raster null mask object
		IMAGE::CELLTYPE m_CellType;
		IMAGE::CELLTYPE m_ParentCellType;
		mutable SIMPLE_ARRAY<UINT8> m_Buffer;
		SIMPLE_ARRAY<UINT8> m_OpacityMap;
		OBJITEM m_ParentObjItem;
		
		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual ERRVALUE v_CopyObject (RVC::OBJECT& DestParentObj, RVC::OBJECT& DestObj, FILTER* filter, COPYFLAGS CopyFlags, const CREDENTIALS* Credentials) const;
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual FILTER* v_GetObjectFilter () const;
		virtual bool v_IsObjectOpen () const;					
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);
			
		//! Implementation of RVC::GEOSPATIAL virtual methods
		virtual void v_GetExtents (DRECT3D &extents) const;
		virtual void v_GetScale (double& xscale, double& yscale) const;
		virtual double v_GetZScale () const;
		virtual double v_GetZOffset () const;
		virtual bool v_HasCapability (int Capability) const;
		virtual void v_SetScale (double xscale, double yscale);
		virtual void v_SetZScale (double zscale);
		virtual void v_SetZOffset (double zoffset);
		
		//! Implementation of RVC::IMAGE virtual methods
		virtual ERRVALUE v_GetBandParms (INT32 BandNum, BANDPARMS& BandParms) const;
		virtual CELLTYPE v_GetCellType () const;
		virtual double v_GetColumnScale () const;
		virtual double v_GetLineScale () const;
		virtual ERRVALUE v_GetNullValue (INT32 BandNum, NULLVALUE& NullValue) const;
		virtual INT32 v_GetNumBands () const;
		virtual INT32 v_GetNumColumns () const;
		virtual INT32 v_GetNumLines () const;
		virtual bool v_HasNullValue (INT32 BandNum) const;
		virtual ERRVALUE v_ReadBand (INT32 Band, INT32 Line, INT32 StartColumn, void *data, INT32 Length, CONVMODE Convert) const;
		virtual ERRVALUE v_ReadColumnBand (INT32 Band, INT32 Line, INT32 StartColumn, void *data, INT32 Length, CONVMODE Convert) const;
		virtual ERRVALUE v_SetCacheRect (const LRECT2D& Rect);
		virtual ERRVALUE v_SetCacheSize (INT32 NumTiles);
		virtual ERRVALUE v_SetCacheSize (CACHEMODE CacheFlags);
		virtual ERRVALUE v_SetNullValue (INT32 BandNum, const NULLVALUE* NullValue);
		virtual ERRVALUE v_WriteBand (INT32 Band, INT32 Line, INT32 StartColumn, const void *data, INT32 Length, CONVMODE Convert);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


//! Convenience filter for raster null mask selection.  
//! Valid null mask raster selection is based on the size of the image to select for
class OPACITYMASK::GETOBJFILTER_ISVALIDOPACITYMASK : public GETOBJFILTER {
	public:
		explicit GETOBJFILTER_ISVALIDOPACITYMASK (
			const RASTER& RastObj
			);
			
		explicit GETOBJFILTER_ISVALIDOPACITYMASK (
			const RASTER::MAKEPARMS& MakeParms
			);
			
		GETOBJFILTER_ISVALIDOPACITYMASK (
			INT32 NumLines,
			INT32 NumColumns
			);
			
	private:
		INT32 m_NumLines;
		INT32 m_NumColumns;
		
		virtual bool v_HasSelectableSubObjects (const RVC::OBJITEM& objitem);
		virtual bool v_IsSelectable (const RVC::OBJITEM& objitem, int position = -1);
	};
	
//! Class to handle values needed to define the characteristics of a opacity mask
class OPACITYMASK::MAKEPARMS {
	public:

		//! Constructor.
		MAKEPARMS (
			IMAGE::CELLTYPE CellType = IMAGE::CELLTYPE_Unsigned_8,
			RASTER::COMPTYPE CompType = RASTER::COMPTYPE_None,
			UINT8 InitialValue = 255
			);

		//! Obtain image cell type.
		//! @return RVC::IMAGE::CELLTYPE for the opacity mask.
		IMAGE::CELLTYPE GetCellType (
			) const { return (m_CellType); }
			
		//! Obtain raster compression type
		//! @return RVC::RASTER::COMPTYPE for the raster.
		RASTER::COMPTYPE GetCompressionType (
			) const { return (m_CompType); }

		//! Get initial value.
		UINT8 GetInitialValue (
			) const { return (m_InitValue); }

		//! Set raster compression values
		void SetCompression (
			RASTER::COMPTYPE CompType			//!< Raster compression type
			);
					
		//! Set image cell type.
		//! Use IMAGE::CELLTYPE_Binary for 1 = Opaque, 0 = Transparent
		//! Use IMAGE::CELLTYPE_Unsigned_8 for 255 = Opaque -> 0 = Transparent
		void SetCellType (
			CELLTYPE CellType
			);

		//! Set initial value.
		//! If not set will make all opaque.
		void SetInitialValue (
			UINT8 InitValue
			) { m_InitValue = InitValue; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CELLTYPE m_CellType;							// Image cell type
		RASTER::COMPTYPE m_CompType;				// Compression type
		UINT8 m_InitValue;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of MAKEPARMS nested class.


//! Object filter for finding OPACITYMASK raster objects
class OPACITYMASK::OBJECT_FILTER : public OBJECT::FILTER {
	public:
		OBJECT_FILTER () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool Filter (const RVC::OBJECT& obj);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif 		// INC_RVC_OPACITYMASK_H

