/**
 * \file rvc/stylepatternbase.h 
 * \brief interface for the RVC::STYLEPATTERN_BASE class.
 *
 * \if NODOC
 * $Id: stylepatternbase.h_v 1.13 2011/12/23 14:10:38 mju Exp $
 *
 * $Log: stylepatternbase.h_v $
 * Revision 1.13  2011/12/23 14:10:38  mju
 * Remove redundant hdr.
 *
 * Revision 1.12  2010/02/24 16:48:09  mju
 * Remove unused specialcolor_UserDefined.
 *
 * Revision 1.11  2008/09/10 13:49:04  mju
 * Fix doxygen warning.
 *
 * Revision 1.10  2007/10/02 20:54:38  dwilliss
 * Account for future posibility of GetNumColors() returning -1, which is how
 * we'll flag RGBA bitmap patterns.
 *
 * Revision 1.9  2007/04/04 17:49:11  scowan
 * Added virtual methods.
 *
 * Revision 1.8  2007/01/24 21:43:27  dwilliss
 * IsEmpty() was only checking the first *byte* of the buffer to see if the
 * name was empty.  On a hi-lo byte order machine, the first byte will almost
 * always be 0 (for Latin1 compatible names anyway), so they all looked "empty"
 *
 * Revision 1.7  2006/11/08 15:23:01  scowan
 * Added comments for special pattern col,ors.
 *
 * Revision 1.6  2006/11/01 21:10:41  scowan
 * Added an isEmpty mehtod.
 *
 * Revision 1.5  2006/09/21 14:30:09  scowan
 * Added necessary include file.
 *
 * Revision 1.4  2006/09/21 14:19:50  scowan
 * Added pattern type to the base.
 *
 * Revision 1.3  2006/09/20 16:51:21  dwilliss
 * SetPattern's buffer should be const
 *
 * Revision 1.2  2006/09/20 14:35:57  scowan
 * Added pattern color enum.
 *
 * Revision 1.1  2006/09/18 17:54:10  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_STYLEPATTERNBASE_H
#define INC_RVC_STYLEPATTERNBASE_H

#ifndef INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef INC_RVC_OBJECTNAME_H
	#include <rvc/objectname.h>
#endif

#ifndef INC_RVC_STYLETYPE_H
	#include <rvc/styletype.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class STYLE;
#endif // GENERATING_DOXYGEN_OUTPUT

//! Base pattern class.
class STYLEPATTERN_BASE {
	public:
	
		enum SPECIALCOLOR {
			SPECIALCOLOR_Transparent = 255,
			SPECIALCOLOR_Variable = 	254
			};

		//! Default constructor used by style object.
		STYLEPATTERN_BASE (
			) : 
			m_PattBuff(0), 
			m_PattHdrSize(sizeof(PATTERNHEADER)), 
			m_PattBuffSize(sizeof(PATTERNHEADER)), 
			m_BuffSizeAlloc(sizeof(PATTERNHEADER))
			{ 
			MmAllocCExc((void**)&m_PattBuff, m_PattBuffSize); 
			}
			
		//! Destructor.
		virtual ~STYLEPATTERN_BASE (
			) { MmFree(m_PattBuff); }

		//! Append color to end of palette.
		//! Note, this will invalidate all derived class ITERATORs.
		ERRVALUE AppendColor (
			const COLOR& color
			);

		//! Clear pattern contents.
		void Clear (
			) { m_PattBuffSize = m_PattHdrSize; memset(m_PattBuff,0,m_PattHdrSize); }

		//! Get pattern name.			
		const RVC::OBJECTNAME& GetName (
			) const { return (*reinterpret_cast<const OBJECTNAME*>(m_PattBuff)); }

		//! Get color at specified palette index.
		const COLOR& GetColor (
			UINT8 idx
			) const { return (GetColors()[idx]); }

		//! Get color palette.
		const COLOR* GetColors (
			) const { return (reinterpret_cast<COLOR*>(m_PattBuff + m_PattHdrSize)); }
			
		//! Get number of user-defined colors in pattern palette.
		//! Note: for RGBA bitmap pattern, GetNumColors() will return -1
		INT32 GetNumColors (
			) const { return (reinterpret_cast<const PATTERNHEADER*>(m_PattBuff)->numcolors); }

		//! Get number of pattern elements.
		INT32 GetNumElements (
			) const { return (reinterpret_cast<const PATTERNHEADER*>(m_PattBuff)->numelements); }

		//! Get the pattern type of this pattern			
		PATTERNTYPE GetPatternType (
			) const { return (m_PattType); }

		//! Get pattern buffer size.
		// XXX make protected when possible.
		INT32 GetSize (
			) const { return (m_PattBuffSize); }

		//! Determine if the pattern is empty.  
		//! @return 'True' if the pattern is empty, 'false' if not
		bool IsEmpty (
			) const { return (v_IsEmpty()); }

		//! Determine if pattern definitions equal, excluding name.
		bool IsEqualDef (
			const STYLEPATTERN_BASE& rhs
			) const {
			if (m_PattBuffSize != rhs.m_PattBuffSize) return (false);
			return (memcmp(m_PattBuff+sizeof(OBJECTNAME),rhs.m_PattBuff+sizeof(OBJECTNAME),m_PattBuffSize-sizeof(OBJECTNAME))==0);
			}

		//! Set color at specified palette index.
		void SetColor (
			UINT8 idx,
			const COLOR& color
			) { if (idx < GetNumColors()) reinterpret_cast<COLOR*>(m_PattBuff + m_PattHdrSize)[idx] = color; }

		//! Set pattern name.
		void SetName (
			const OBJECTNAME& name
			) { *reinterpret_cast<OBJECTNAME*>(m_PattBuff) = name; }

		//! This method was added for use only in HATCHPATTERN because it has embedded LINEPATTERN
		ERRVALUE SetPattern (
			const void* SrcPattBuff,
			INT32 PattBuffSize,
			INT32 PattHdrSize
			);

		//! To support legacy code, for now.
		// XXX delete when drawing code updated.
		operator const void* (
			) const { return (m_PattBuff); }

			
	protected:

		//! Information common to all patterns / symbols.
		struct PATTERNHEADER {
			RVC::OBJECTNAME name;					//!< Name
			INT16 numelements;						//!< Number of elements used to define pattern
			INT16 numcolors;							//!< Number of colors used in pattern
			};

		//! Constructor used by derived classes.
		STYLEPATTERN_BASE (
			PATTERNTYPE PattType,
			int PattHdrSize
			) : 
			m_PattBuff(0), 
			m_PattHdrSize(PattHdrSize), 
			m_PattBuffSize(PattHdrSize), 
			m_BuffSizeAlloc(PattHdrSize),
			m_PattType(PattType)
			{ 
			MmAllocCExc((void**)&m_PattBuff, m_PattBuffSize); 
			}
			
		//! Copy constructor.
		STYLEPATTERN_BASE (const STYLEPATTERN_BASE& rhs);

		//! Assignment.
		STYLEPATTERN_BASE& operator= (const STYLEPATTERN_BASE& rhs);
		
		//! Delete colors in specified set.
		void DeleteColorSet (
			const UINT8 *set					//!< Byte array with entries set to non-zero to delete, zero to retain
			);

		//! Delete from pattern buffer at specified location.
		void DeleteFromBuffer (
			INT32 offset,
			INT32 numbytes
			);

		//! Get const pointer to pattern buffer.
		const UINT8* GetPattBuffer (
			) const { return (m_PattBuff); }

		//! Get non-const pointer to pattern buffer.
		UINT8* GetPattBuffer (
			) { return (m_PattBuff); }

		//! Get const pointer to first pattern 'element'.
		const UINT8* GetFirstElemPtr (
			) const { return (m_PattBuff+m_PattHdrSize+((GetNumColors()>0)?GetNumColors():0)*sizeof(COLOR)); }

		//! Get non-const pointer to first pattern 'element'.
		UINT8* GetFirstElemPtr (
			) { return (m_PattBuff+m_PattHdrSize+((GetNumColors()>0)?GetNumColors():0)*sizeof(COLOR)); }

		//! Insert space into pattern buffer at specified location
		ERRVALUE InsertIntoBuffer (
			INT32 offset,
			INT32 numbytes,
			const void *data = 0
			);
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT8* m_PattBuff;
		INT32 m_PattHdrSize;
		INT32 m_PattBuffSize;
		mutable SIMPLE_ARRAY<UINT32> m_OffsetArray;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_BuffSizeAlloc;
		PATTERNTYPE m_PattType;

		//! If the number of elements is zero, then it is an empty pattern.
		//! STYLEPATTERN_BITMAP has its own test, since it has no elements to speak of
		virtual bool v_IsEmpty (
			) const { return (GetNumElements() == 0); }
			
		friend class STYLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif 	// INC_RVC_STYLEPATTERNBASE_H
