/**
 * \file soa/wmsconnection.h
 * \brief SOA::WMS::CONNECTION definition
 *
 * \if NODOC
 * $Id: wmsconnection.h_v 1.9 2012/11/15 16:14:02 ldyas Exp $
 *
 * $Log: wmsconnection.h_v $
 * Revision 1.9  2012/11/15 16:14:02  ldyas
 * *** empty log message ***
 *
 * Revision 1.8  2012/11/01 14:54:44  ldyas
 * allowing override of get map
 *
 * Revision 1.7  2011/03/18 20:53:14  ldyas
 * new auth methods.
 *
 * Revision 1.6  2011/03/16 01:41:58  ldyas
 * added authorization dialog setter
 *
 * Revision 1.5  2007/04/12 20:10:14  mju
 * Add dlgparent parm to doGetCapabilities.
 *
 * Revision 1.4  2007/02/12 21:14:15  mju
 * Add initRequest.
 *
 * Revision 1.2  2007/01/25 22:12:07  mju
 * Add doGetCapabilities w/o serverURL.
 *
 * Revision 1.1  2007/01/19 18:40:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_WMSCONNECTION_H
#define	INC_SOA_WMSCONNECTION_H

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

namespace SPATREF {
	class COORDREFSYS;
	}
namespace SPATMOD {
	namespace IMAGE {
		class SOURCE_HTML;
		}
	}
struct COLOR;
class MIMETYPE;

namespace SOA {
namespace WMS {

class CAPABILITIES;

//=====================================================================================================================

//! Container for information about connection to a WMS server.
//! Includes Server URL and capabilities, and provides methods to send service requests.
class CONNECTION {
	public:

		//! Constructor.
		CONNECTION ();
		
		//! Copy constructor.
		CONNECTION (const CONNECTION&);

		//! Destructor.
		~CONNECTION ();

		//! Assignment.
		CONNECTION& operator= (const CONNECTION&);

		//! Build "GetFeatureInfo" request URL string
		ERRVALUE BuildGetFeatureInfoRequest (
			MISTRING& RequestURL,									//!< Request URL returned
			const MISTRINGLIST& LayerNameList,					//!< List of layers to request
			const MISTRINGLIST& LayerStyleList,					//!< Corresponding layer style list
			const SPATREF::COORDREFSYS& CoordRefSys,			//!< CRS for bounding box and rendering
			const DRECT2D& BoundingBox,							//!< Bounding box
			INT32 TgtWidth,											//!< Target width in pixels
			INT32 TgtHeight,											//!< Target height in pixels
			const MIMETYPE& ImageFormat,							//!< Image format MIME type
			const MISTRINGLIST& QueryLayerList,					//!< List of layers to query
			const MIMETYPE& InfoFormat,							//!< Format for returning feature info
			INT32 PixelColumn,										//!< Column coordinate of position to query
			INT32 PixelRow,											//!< Row coordinate of position to query
			INT32 FeatureCount = 1									//!< Feature count
			) const;

		//! Build "GetMap" request URL string.
		ERRVALUE BuildGetMapRequest (
			MISTRING& RequestURL,									//!< Request URL returned
			const MISTRINGLIST& LayerNameList,					//!< List of layers to request
			const MISTRINGLIST& LayerStyleList,					//!< Corresponding layer style list
			const SPATREF::COORDREFSYS& CoordRefSys,			//!< CRS for bounding box and rendering
			const DRECT2D& BoundingBox,							//!< Bounding box
			INT32 TgtWidth,											//!< Target width in pixels
			INT32 TgtHeight,											//!< Target height in pixels
			const MIMETYPE& Format,									//!< Format MIME type
			const COLOR *pColorBG = 0								//!< Optional background color
			) const;

		//! Send "GetCapabilities" request to specified server.
		//! If matches previous setting will not request again.
		ERRVALUE DoGetCapabilities (
			const MISTRING& ServerURL,
			bool synchronous,
			MDLGPARENT dlgparent = 0								//!< Parent for status dialog if want to show
			);

		//! Send "GetCapabilities" request to previously specified server.
		//! If already have capabilities will not request again.
		ERRVALUE DoGetCapabilities (
			bool synchronous,
			MDLGPARENT dlgparent = 0								//!< Parent for status dialog if want to show
			);

		//! Get capabilities as returned from server.
		//! If an asynchronous request for capabilities was made and which may still be open
		//! then must call HasCapabilities first and only call this method if returns true.
		const WMS::CAPABILITIES& GetCapabilities (
			) const;

		//! Get URL for server.
		const MISTRING& GetServerURL (
			) const { return (m_ServerURL); }

		const MISTRING& GetUsername (
			) const {
			return m_Username;
			}

		const MISTRING& GetPassword (
			) const {
			return m_Password;
			}

		bool GetUseServerUrl (
			) const { return m_UseServerUrl; }

		//! Determine if have capabilities available.
		bool HasCapabilities (
			) const { return (m_HasCapabilities); }

		//! Set URL for server.
		//! If does not match existing then capabilities will be discarded.
		void SetServerURL (
			const MISTRING& ServerURL
			);
		void SetDoAuthDialog (
			const bool doAuth
			){ m_DoAuthorization = doAuth; }

		void SetUseServerUrl (
			const bool use
			){ m_UseServerUrl = use; }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		MISTRING m_ServerURL;
		WMS::CAPABILITIES *m_pCapabilities;
		bool m_HasCapabilities;
		bool m_DoAuthorization;
		bool m_UseServerUrl;
		MISTRING m_Username;
		MISTRING m_Password;


		static void AppendNameListRequest (
			MISTRING& RequestURL,
			const MISTRINGLIST& NameList
			);

		ERRVALUE AppendGetMapRequestPart (
			MISTRING& RequestURL,									//!< Request URL returned
			const MISTRINGLIST& LayerNameList,					//!< List of layers to request
			const MISTRINGLIST& LayerStyleList,					//!< Corresponding layer style list
			const SPATREF::COORDREFSYS& CoordRefSys,			//!< CRS for bounding box and rendering
			const DRECT2D& BoundingBox,							//!< Bounding box
			INT32 TgtWidth,											//!< Target width in pixels
			INT32 TgtHeight,											//!< Target height in pixels
			const MIMETYPE& Format,									//!< Format MIME type
			const COLOR *pColorBG = 0								//!< Optional background color
			) const;

		void InitRequest (
			MISTRING& RequestURL										//!< Request URL returned
			) const;

		#endif
	};

//=====================================================================================================================

}	// End namespace WMS
}	// End namespace SOA

#endif	// INC_SOA_WMSCONNECTION_H
