/**
 * \file spatmod/imagetargetjpeg.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_JPEG definition.
 *
 * \if NODOC
 *
 * $Id: imagetargetjpeg.h_v 1.7 2011/10/11 14:52:22 mju Exp $
 *
 * $Log: imagetargetjpeg.h_v $
 * Revision 1.7  2011/10/11 14:52:22  mju
 * Remove unneeded hdr.
 *
 * Revision 1.6  2009/08/13 16:46:27  mju
 * Base factory not nested.
 *
 * Revision 1.5  2009/08/13 15:38:24  mju
 * Add factory.
 *
 * Revision 1.4  2007/01/26 23:27:23  dfriberg
 * Add virtual methods to be called when writing one output file per tile
 *
 * Revision 1.3  2006/12/07 22:47:39  mju
 * Support memory buffer output.
 *
 * Revision 1.2  2005/12/16 16:09:07  mju
 * Change enum name.
 *
 * Revision 1.1  2005/12/12 21:45:35  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETJPEG_H
#define	INC_SPATMOD_IMAGETARGETJPEG_H

#ifndef	INC_SPATMOD_IMAGETARGETFILE_H
#include <spatmod/imagetargetfile.h>
#endif

class JPEGEXT;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! JPEG image target.
//! Supports DATATYPE of UINT8 only.
//! Supports PIXEL::TYPE of Single, Grayscale, RGB, CMYK only.
class TARGET_JPEG : public TARGET_FILE {
	public:

		//! Factory class for constructing TARGET instance.
		class FACTORY : public FACTORY_TARGET_FILE {
			public:

				//! Standard constructor.
				explicit FACTORY (	
					UINT16 quality = 75						//!< Compression quality
					);

				//! Copy constructor.
				FACTORY (const FACTORY& rhs);

				//! Destructor.
				virtual ~FACTORY ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				UINT16 m_Quality;

				// FACTORY_TARGET_FILE overrides.
				virtual FACTORY_TARGET_FILE * v_Clone () const;
				virtual TARGET_FILE * v_ConstructTarget (STAGE *pInputStage, const FILEPATH& filepath, GEOFORMATS GeoFormats) const;

				// Unimplemented.
				FACTORY& operator= (const FACTORY& rhs);
				#endif
			};

		//! Construct target with specified path.
		TARGET_JPEG (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath,					//!< Path to target file
			UINT16 quality = 75,							//!< Compression quality
			GEOFORMATS GeoFormats = GEOFORMAT_None	//!< Georeference formats
			);

		//! Construct for storage in memory buffer.
		TARGET_JPEG (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			UINT16 quality = 75,							//!< Compression quality
			GEOFORMATS GeoFormats = GEOFORMAT_None	//!< Georeference formats
			);

		//! Destructor.
		virtual ~TARGET_JPEG ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		JPEGEXT *m_pWriter;
		UINT16 m_Quality;
		bool m_UseMemory;
		SIMPLE_ARRAY<UINT8> m_RowBuf;

		// TARGET overrides.
		virtual ERRVALUE v_FillTileRows(INT32, INT32);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_TileBegin (const FILEPATH& FilePath, const LRECT2D& SubSection);
		virtual ERRVALUE v_TileEnd ();
		virtual ERRVALUE v_WriteSubSection (const SECTION& SubSection, const BUFFER& buffer);

		// TARGET_FILE overrides.
		virtual void* v_DetachBuffer ();
		virtual const void* v_GetBuffer () const;
		virtual INT64 v_GetBufferSize () const;

		// Unimplemented
		TARGET_JPEG (const TARGET_JPEG&);
		TARGET_JPEG& operator= (const TARGET_JPEG&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETJPEG_H

