/**
 * \file spatmod/imagetargettilesetfile.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_TILESET_FILE definition.
 *
 * \if NODOC
 * $Id: imagetargettilesetfile.h_v 1.20 2012/09/18 16:13:11 bcolombe Exp $
 *
 * $Log: imagetargettilesetfile.h_v $
 * Revision 1.20  2012/09/18 16:13:11  bcolombe
 * Add no write test mode
 *
 * Revision 1.19  2012/04/27 16:37:20  bcolombe
 * Use scale numerator,denom
 *
 * Revision 1.18  2012/04/26 16:59:56  bcolombe
 * Remove precision, just use scale
 *
 * Revision 1.17  2012/04/20 21:27:15  bcolombe
 * Changes for min/max value
 *
 * Revision 1.16  2012/03/29 17:38:07  bcolombe
 * Add precision
 *
 * Revision 1.15  2012/03/15 15:35:17  bcolombe
 * Add scale to meters
 *
 * Revision 1.14  2012/01/30 20:22:19  bcolombe
 * Add terrain pixel value min & max
 *
 * Revision 1.13  2010/09/20 16:10:13  bcolombe
 * Added MakeTileMap
 *
 * Revision 1.12  2010/06/03 22:47:45  bcolombe
 * Added UsePalette
 *
 * Revision 1.11  2010/04/02 22:24:43  bcolombe
 * Removed HasTarget/SourceTiles methods
 *
 * Revision 1.10  2010/04/01 20:27:49  bcolombe
 * Removed m_TargetTileFolderBasePath
 *
 * Revision 1.9  2010/02/19 20:43:33  bcolombe
 * Add HasSourceTiles,HasTargetTile that takes path
 *
 * Revision 1.8  2010/02/18 23:53:00  bcolombe
 * Added CreateCatalogExcludeFile
 *
 * Revision 1.7  2010/01/25 15:45:55  bcolombe
 * Add Get/SetCreateAdditionalFiles to control whether PushEnd creates TSD & HTML files
 *
 * Revision 1.6  2009/12/21 21:34:56  bcolombe
 * Added SetTargetTielFolderBasePath
 *
 * Revision 1.5  2009/12/07 21:03:55  bcolombe
 * Added HasTargetTile and HasSourceTiles
 *
 * Revision 1.4  2009/08/28 08:31:21  bcolombe
 * Add v_FindTileFile
 *
 * Revision 1.3  2009/08/13 17:20:23  mju
 * Add ctor using full tileset spec.
 *
 * Revision 1.2  2009/08/06 17:01:35  mju
 * *** empty log message ***
 *
 * Revision 1.1  2009/08/05 17:43:03  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETTILESETFILE_H
#define	INC_SPATMOD_IMAGETARGETTILESETFILE_H

#ifndef	INC_SPATMOD_IMAGETARGETTILESET_H
#include <spatmod/imagetargettileset.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class TARGET_FILE;

//=====================================================================================================================

//! Target base class for outputting tileset as files.
class TARGET_TILESET_FILE : public TARGET_TILESET {
	public:

		//! Destructor.
		virtual ~TARGET_TILESET_FILE ();

		//! Get whether to create addition files (TSD, html, etc).
		bool GetCreateAdditionalFiles (
			) const { return (m_CreateAdditionalFiles); }

		//! Get whether to delete files after zipping.
		bool GetDeleteAfterZip (
			) const { return (m_DeleteAfterZip); }

		//! Get whether to make tile map
		bool GetMakeTileMap (
			) const { return (m_MakeTileMap); }

		//! Get whether set to no write test mode
		bool GetNoWrite (
			) const { return (m_NoWrite); }

		//! Get tile folder base path.
		const FILEPATH& GetTileFolderBasePath (
			) const { return (m_TileFolderBasePath); }

		//! Get whether to use palette for output tiles
		bool GetUsePalette (
			) const { return (m_UsePalette); }

		//! Create catalog exclude file in tiles folder
		ERRVALUE CreateCatalogExcludeFile ();

		//! Scale to meters
		double GetScale () const { return (m_Scale); }

		INT32 GetScaleNumerator () const { return (m_ScaleNumerator); }

		INT32 GetScaleDenominator () const { return (m_ScaleDenominator); }

		INT64 GetPrescaleOffset () const { return (m_PrescaleOffset); }

		INT64 GetPostscaleOffset () const { return (m_PostscaleOffset); }

		double GetPixelValueMin () const { return (m_PixelValueMin); }

		double GetPixelValueMax () const { return (m_PixelValueMax); }

		//! Get whether to zip output.
		bool GetZipOutput (
			) const { return (m_ZipOutput); }

		//! Set whether to create addition files (TSD, html, etc).
		void SetCreateAdditionalFiles (
			bool CreateAdditionalFiles
			) { m_CreateAdditionalFiles = CreateAdditionalFiles; }

		//! Set whether to make tile map
		void SetMakeTileMap (
			bool MakeTileMap
			) { m_MakeTileMap = MakeTileMap; }

		//! Set no write test mode
		void SetNoWrite (
			bool NoWrite
			) { m_NoWrite = NoWrite; }

		void SetPixelValueRange (
			double PixelValueMin,
			double PixelValueMax
			) { m_PixelValueMin = PixelValueMin; m_PixelValueMax = PixelValueMax; }

		//! Scale to meters
		void SetScale (
			double Scale
			) { m_Scale = Scale; }

		void SetScaleRational (
			INT32 Numerator,
			INT32 Denominator
			) { m_ScaleNumerator = Numerator; m_ScaleDenominator = Denominator; m_Scale = (double)Numerator/(double)Denominator; }

		void SetPrescaleOffset (
			INT64 offset
			) { m_PrescaleOffset = offset; }

		void SetPostscaleOffset (
			INT64 offset
			) { m_PostscaleOffset = offset; }

		//! Set tile folder base path.
		void SetTileFolderBasePath (
			const FILEPATH& filepath
			) { m_TileFolderBasePath = filepath; }

		//! Set to use a color palette for output tiles
		//! Not supported for JPEG
		void SetUsePalette (
			bool UsePalette = true
			) { m_UsePalette = UsePalette; }

		//! Zip the output
		void SetZipOutput (
			bool ZipOutput = true,
			bool DeleteAfterZip = false
			) { m_ZipOutput = ZipOutput; m_DeleteAfterZip = DeleteAfterZip; }

	protected:

		//! Construct with TILESET specification.
		TARGET_TILESET_FILE (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const IMAGE::TILESET& tileset				//!< Complete tileset specification
			);

		//! Construct with tilesize, profile and format.
		TARGET_TILESET_FILE (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			INT32 TileSize,								//!< Tile Size
			IMAGE::TILESET::PROFILE profile,			//!< Tileset profile
			const IMAGE::FORMAT format					//!< Image tile file format
			);

		//! Construct with tilesize and profile.
		TARGET_TILESET_FILE (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			INT32 TileSize,								//!< Tile Size
			IMAGE::TILESET::PROFILE profile			//!< Tileset profile
			);

		//! Create IMAGE::TARGET_FILE for output of single tile using specified format.
		ERRVALUE CreateTargetFile (
			IMAGE::STAGE *pStageInput,					//!< Source containing single tile
			const FILEPATH& filepath,					//!< Tile file path
			const IMAGE::FORMAT& format,				//!< File format
			IMAGE::TARGET_FILE *& pTargetFile		//!< Pointer returned, will be deleted by caller
			) const;

		//! Create IMAGE::TARGET_FILE for output of single tile, format based on whether has null cells.
		ERRVALUE CreateTargetFile (
			IMAGE::STAGE *pStageInput,					//!< Source containing single tile
			const FILEPATH& filepath,					//!< Tile file path
			bool HasNullCells,							//!< Set if tile has null cells
			IMAGE::TARGET_FILE *& pTargetFile		//!< Pointer returned, will be deleted by caller
			) const;

		//! Find tile file if exists.
		//! @return Pointer to format for tile file if found, NULL if not found.
		const IMAGE::FORMAT* FindTileFile (
			FILEPATH& filepath,							//!< Tile file path returned
			const LPOINT3D& TileIdx						//!< Tile index (x = column, y = row, z = zoom level)
			) const { return (v_FindTileFile(filepath,TileIdx)); }

		//! Generate path to tile file.
		ERRVALUE GenerateTileFilePath (
			FILEPATH& filepath,							//!< Tile file path returned
			const LPOINT3D& TileIdx,					//!< Tile index (x = column, y = row, z = zoom level)
			bool HasNullCells,							//!< Set if tile has null cells
			bool CreateFolder								//!< Create folder if does not exist
			) const { return (v_GenerateTileFilePath(filepath,TileIdx,HasNullCells,CreateFolder)); }

		//! Find tile file if exists.
		//! @return Pointer to format for tile file if found, NULL if not found.
		//! Default implementation passes TileFolderBasePath to TILESET's FindTileFile
		virtual const IMAGE::FORMAT* v_FindTileFile (
			FILEPATH& filepath,							//!< Tile file path returned
			const LPOINT3D& TileIdx						//!< Tile index (x = column, y = row, z = zoom level)
			) const;

		//! Generate path to tile file.
		//! Default implementation uses structure defined in TILESET, with file extension based on FORMAT::SPEC.
		//! If more than one format is supported then will use first suitable based on value of HasNullCells.
		virtual ERRVALUE v_GenerateTileFilePath (
			FILEPATH& filepath,							//!< Tile file path returned
			const LPOINT3D& TileIdx,					//!< Tile index (x = column, y = row, z = zoom level)
			bool HasNullCells,							//!< Set if tile has null cells
			bool CreateFolder								//!< Create folder if does not exist
			) const;

		//! Write tile.
		//! Uses GenerateTileFilePath and CreateTargetFile.
		virtual ERRVALUE v_WriteTile (
			SOURCE_MEMORY& SourceTile,					//!< Source for accessing data to write tile
			const LPOINT3D& TileIdx,					//!< Tile index (x = column, y = row, z = zoom level)
			bool HasNullCells								//!< True if tile has any null cells
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILEPATH m_TileFolderBasePath;
		bool m_ZipOutput;
		bool m_DeleteAfterZip;
		bool m_CreateAdditionalFiles;
		bool m_UsePalette;
		bool m_MakeTileMap;
		bool m_NoWrite;
		double m_PixelValueMin;
		double m_PixelValueMax;
		double m_Scale;
		INT32 m_ScaleNumerator;
		INT32 m_ScaleDenominator;
		INT64 m_PrescaleOffset;
		INT64 m_PostscaleOffset;

		// Unimplemented
		TARGET_TILESET_FILE (const TARGET_TILESET_FILE&);
		TARGET_TILESET_FILE& operator= (const TARGET_TILESET_FILE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Create IMAGE::TARGET_FILE for output of single tile.
		//! Must be overridden if default v_WriteTile is used as default implementation simply returns EProgramError.
		virtual ERRVALUE v_CreateTargetFile (
			IMAGE::STAGE *pStageInput,					//!< Source containing single tile
			const FILEPATH& filepath,					//!< Tile file path
			const IMAGE::FORMAT& format,				//!< File format
			IMAGE::TARGET_FILE *& pTargetFile		//!< Pointer returned, will be deleted by caller
			) const;

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETTILESETFILE_H

